@@ -11,60 +11,51 @@ import fnmatch
 import functools
 import json
 import os
+import queue
+import re
 import shutil
 import stat
 import sys
 import tarfile
 import tempfile
+import textwrap
 import threading
 import time
-import yaml
+import typing as t
 
 from collections import namedtuple
 from contextlib import contextmanager
-from distutils.version import LooseVersion
 from hashlib import sha256
 from io import BytesIO
+from importlib.metadata import distribution
 from itertools import chain
-from yaml.error import YAMLError
-
-# NOTE: Adding type ignores is a hack for mypy to shut up wrt bug #1153
-try:
-    import queue  # type: ignore[import]
-except ImportError:  # Python 2
-    import Queue as queue  # type: ignore[import,no-redef]
 
 try:
-    # NOTE: It's in Python 3 stdlib and can be installed on Python 2
-    # NOTE: via `pip install typing`. Unnecessary in runtime.
-    # NOTE: `TYPE_CHECKING` is True during mypy-typecheck-time.
-    from typing import TYPE_CHECKING
+    from packaging.requirements import Requirement as PkgReq
 except ImportError:
-    TYPE_CHECKING = False
+    class PkgReq:  # type: ignore[no-redef]
+        pass
 
-if TYPE_CHECKING:
-    from typing import Dict, Iterable, List, Optional, Text, Union
-    if sys.version_info[:2] >= (3, 8):
-        from typing import Literal
-    else:  # Python 2 + Python 3.4-3.7
-        from typing_extensions import Literal
+    HAS_PACKAGING = False
+else:
+    HAS_PACKAGING = True
 
-    from ansible.galaxy.api import GalaxyAPI
+if t.TYPE_CHECKING:
     from ansible.galaxy.collection.concrete_artifact_manager import (
         ConcreteArtifactsManager,
     )
 
-    ManifestKeysType = Literal[
+    ManifestKeysType = t.Literal[
         'collection_info', 'file_manifest_file', 'format',
     ]
-    FileMetaKeysType = Literal[
+    FileMetaKeysType = t.Literal[
         'name',
         'ftype',
         'chksum_type',
         'chksum_sha256',
         'format',
     ]
-    CollectionInfoKeysType = Literal[
+    CollectionInfoKeysType = t.Literal[
         # collection meta:
         'namespace', 'name', 'version',
         'authors', 'readme',
@@ -77,26 +68,14 @@ if TYPE_CHECKING:
         # files meta:
         FileMetaKeysType,
     ]
-    ManifestValueType = Dict[
-        CollectionInfoKeysType,
-        Optional[
-            Union[
-                int, str,  # scalars, like name/ns, schema version
-                List[str],  # lists of scalars, like tags
-                Dict[str, str],  # deps map
-            ],
-        ],
-    ]
-    CollectionManifestType = Dict[ManifestKeysType, ManifestValueType]
-    FileManifestEntryType = Dict[FileMetaKeysType, Optional[Union[str, int]]]
-    FilesManifestType = Dict[
-        Literal['files', 'format'],
-        Union[List[FileManifestEntryType], int],
-    ]
+    ManifestValueType = t.Dict[CollectionInfoKeysType, t.Union[int, str, t.List[str], t.Dict[str, str], None]]
+    CollectionManifestType = t.Dict[ManifestKeysType, ManifestValueType]
+    FileManifestEntryType = t.Dict[FileMetaKeysType, t.Union[str, int, None]]
+    FilesManifestType = t.Dict[t.Literal['files', 'format'], t.Union[t.List[FileManifestEntryType], int]]
 
 import ansible.constants as C
 from ansible.errors import AnsibleError
-from ansible.galaxy import get_collections_galaxy_meta_info
+from ansible.galaxy.api import GalaxyAPI
 from ansible.galaxy.collection.concrete_artifact_manager import (
     _consume_file,
     _download_file,
@@ -105,23 +84,40 @@ from ansible.galaxy.collection.concrete_artifact_manager import (
     _tarfile_extract,
 )
 from ansible.galaxy.collection.galaxy_api_proxy import MultiGalaxyAPIProxy
-from ansible.galaxy.dependency_resolution import (
-    build_collection_dependency_resolver,
+from ansible.galaxy.collection.gpg import (
+    run_gpg_verify,
+    parse_gpg_errors,
+    get_signature_from_source,
+    GPG_ERROR_MAP,
 )
+try:
+    from ansible.galaxy.dependency_resolution import (
+        build_collection_dependency_resolver,
+    )
+    from ansible.galaxy.dependency_resolution.errors import (
+        CollectionDependencyResolutionImpossible,
+        CollectionDependencyInconsistentCandidate,
+    )
+    from ansible.galaxy.dependency_resolution.providers import (
+        RESOLVELIB_VERSION,
+        RESOLVELIB_LOWERBOUND,
+        RESOLVELIB_UPPERBOUND,
+    )
+except ImportError:
+    HAS_RESOLVELIB = False
+else:
+    HAS_RESOLVELIB = True
+
 from ansible.galaxy.dependency_resolution.dataclasses import (
     Candidate, Requirement, _is_installed_collection_dir,
 )
-from ansible.galaxy.dependency_resolution.errors import (
-    CollectionDependencyResolutionImpossible,
-    CollectionDependencyInconsistentCandidate,
-)
 from ansible.galaxy.dependency_resolution.versioning import meets_requirements
 from ansible.module_utils.six import raise_from
 from ansible.module_utils._text import to_bytes, to_native, to_text
+from ansible.module_utils.common.yaml import yaml_dump
 from ansible.utils.collection_loader import AnsibleCollectionRef
 from ansible.utils.display import Display
 from ansible.utils.hashing import secure_hash, secure_hash_s
-from ansible.utils.version import SemanticVersion
 
 
 display = Display()
@@ -131,6 +127,48 @@ MANIFEST_FILENAME = 'MANIFEST.json'
 
 ModifiedContent = namedtuple('ModifiedContent', ['filename', 'expected', 'installed'])
 
+SIGNATURE_COUNT_RE = r"^(?P<strict>\+)?(?:(?P<count>\d+)|(?P<all>all))$"
+
+
+class CollectionSignatureError(Exception):
+    def __init__(self, reasons=None, stdout=None, rc=None, ignore=False):
+        self.reasons = reasons
+        self.stdout = stdout
+        self.rc = rc
+        self.ignore = ignore
+
+        self._reason_wrapper = None
+
+    def _report_unexpected(self, collection_name):
+        return (
+            f"Unexpected error for '{collection_name}': "
+            f"GnuPG signature verification failed with the return code {self.rc} and output {self.stdout}"
+        )
+
+    def _report_expected(self, collection_name):
+        header = f"Signature verification failed for '{collection_name}' (return code {self.rc}):"
+        return header + self._format_reasons()
+
+    def _format_reasons(self):
+        if self._reason_wrapper is None:
+            self._reason_wrapper = textwrap.TextWrapper(
+                initial_indent="    * ",  # 6 chars
+                subsequent_indent="      ",  # 6 chars
+            )
+
+        wrapped_reasons = [
+            '\n'.join(self._reason_wrapper.wrap(reason))
+            for reason in self.reasons
+        ]
+
+        return '\n' + '\n'.join(wrapped_reasons)
+
+    def report(self, collection_name):
+        if self.reasons:
+            return self._report_expected(collection_name)
+
+        return self._report_unexpected(collection_name)
+
 
 # FUTURE: expose actual verify result details for a collection on this object, maybe reimplement as dataclass on py3.8+
 class CollectionVerifyResult:
@@ -142,7 +180,7 @@ class CollectionVerifyResult:
 def verify_local_collection(
         local_collection, remote_collection,
         artifacts_manager,
-):  # type: (Candidate, Optional[Candidate], ConcreteArtifactsManager) -> CollectionVerifyResult
+):  # type: (Candidate, t.Optional[Candidate], ConcreteArtifactsManager) -> CollectionVerifyResult
     """Verify integrity of the locally installed collection.
 
     :param local_collection: Collection being checked.
@@ -162,16 +200,64 @@ def verify_local_collection(
         format(path=to_text(local_collection.src)),
     )
 
-    modified_content = []  # type: List[ModifiedContent]
+    modified_content = []  # type: list[ModifiedContent]
 
     verify_local_only = remote_collection is None
-    if verify_local_only:
-        # partial away the local FS detail so we can just ask generically during validation
-        get_json_from_validation_source = functools.partial(_get_json_from_installed_dir, b_collection_path)
-        get_hash_from_validation_source = functools.partial(_get_file_hash, b_collection_path)
 
+    # partial away the local FS detail so we can just ask generically during validation
+    get_json_from_validation_source = functools.partial(_get_json_from_installed_dir, b_collection_path)
+    get_hash_from_validation_source = functools.partial(_get_file_hash, b_collection_path)
+
+    if not verify_local_only:
+        # Compare installed version versus requirement version
+        if local_collection.ver != remote_collection.ver:
+            err = (
+                "{local_fqcn!s} has the version '{local_ver!s}' but "
+                "is being compared to '{remote_ver!s}'".format(
+                    local_fqcn=local_collection.fqcn,
+                    local_ver=local_collection.ver,
+                    remote_ver=remote_collection.ver,
+                )
+            )
+            display.display(err)
+            result.success = False
+            return result
+
+    manifest_file = os.path.join(to_text(b_collection_path, errors='surrogate_or_strict'), MANIFEST_FILENAME)
+    signatures = list(local_collection.signatures)
+    if verify_local_only and local_collection.source_info is not None:
+        signatures = [info["signature"] for info in local_collection.source_info["signatures"]] + signatures
+    elif not verify_local_only and remote_collection.signatures:
+        signatures = list(remote_collection.signatures) + signatures
+
+    keyring_configured = artifacts_manager.keyring is not None
+    if not keyring_configured and signatures:
+        display.warning(
+            "The GnuPG keyring used for collection signature "
+            "verification was not configured but signatures were "
+            "provided by the Galaxy server. "
+            "Configure a keyring for ansible-galaxy to verify "
+            "the origin of the collection. "
+            "Skipping signature verification."
+        )
+    elif keyring_configured:
+        if not verify_file_signatures(
+            local_collection.fqcn,
+            manifest_file,
+            signatures,
+            artifacts_manager.keyring,
+            artifacts_manager.required_successful_signature_count,
+            artifacts_manager.ignore_signature_errors,
+        ):
+            result.success = False
+            return result
+        display.vvvv(f"GnuPG signature verification succeeded, verifying contents of {local_collection}")
+
+    if verify_local_only:
         # since we're not downloading this, just seed it with the value from disk
         manifest_hash = get_hash_from_validation_source(MANIFEST_FILENAME)
+    elif keyring_configured and remote_collection.signatures:
+        manifest_hash = get_hash_from_validation_source(MANIFEST_FILENAME)
     else:
         # fetch remote
         b_temp_tar_path = (  # NOTE: AnsibleError is raised on URLError
@@ -188,20 +274,6 @@ def verify_local_collection(
         get_json_from_validation_source = functools.partial(_get_json_from_tar_file, b_temp_tar_path)
         get_hash_from_validation_source = functools.partial(_get_tar_file_hash, b_temp_tar_path)
 
-        # Compare installed version versus requirement version
-        if local_collection.ver != remote_collection.ver:
-            err = (
-                "{local_fqcn!s} has the version '{local_ver!s}' but "
-                "is being compared to '{remote_ver!s}'".format(
-                    local_fqcn=local_collection.fqcn,
-                    local_ver=local_collection.ver,
-                    remote_ver=remote_collection.ver,
-                )
-            )
-            display.display(err)
-            result.success = False
-            return result
-
         # Verify the downloaded manifest hash matches the installed copy before verifying the file manifest
         manifest_hash = get_hash_from_validation_source(MANIFEST_FILENAME)
         _verify_file_hash(b_collection_path, MANIFEST_FILENAME, manifest_hash, modified_content)
@@ -219,11 +291,46 @@ def verify_local_collection(
     _verify_file_hash(b_collection_path, file_manifest_filename, expected_hash, modified_content)
     file_manifest = get_json_from_validation_source(file_manifest_filename)
 
+    collection_dirs = set()
+    collection_files = {
+        os.path.join(b_collection_path, b'MANIFEST.json'),
+        os.path.join(b_collection_path, b'FILES.json'),
+    }
+
     # Use the file manifest to verify individual file checksums
     for manifest_data in file_manifest['files']:
+        name = manifest_data['name']
+
         if manifest_data['ftype'] == 'file':
+            collection_files.add(
+                os.path.join(b_collection_path, to_bytes(name, errors='surrogate_or_strict'))
+            )
             expected_hash = manifest_data['chksum_%s' % manifest_data['chksum_type']]
-            _verify_file_hash(b_collection_path, manifest_data['name'], expected_hash, modified_content)
+            _verify_file_hash(b_collection_path, name, expected_hash, modified_content)
+
+        if manifest_data['ftype'] == 'dir':
+            collection_dirs.add(
+                os.path.join(b_collection_path, to_bytes(name, errors='surrogate_or_strict'))
+            )
+
+    # Find any paths not in the FILES.json
+    for root, dirs, files in os.walk(b_collection_path):
+        for name in files:
+            full_path = os.path.join(root, name)
+            path = to_text(full_path[len(b_collection_path) + 1::], errors='surrogate_or_strict')
+
+            if full_path not in collection_files:
+                modified_content.append(
+                    ModifiedContent(filename=path, expected='the file does not exist', installed='the file exists')
+                )
+        for name in dirs:
+            full_path = os.path.join(root, name)
+            path = to_text(full_path[len(b_collection_path) + 1::], errors='surrogate_or_strict')
+
+            if full_path not in collection_dirs:
+                modified_content.append(
+                    ModifiedContent(filename=path, expected='the directory does not exist', installed='the directory exists')
+                )
 
     if modified_content:
         result.success = False
@@ -245,8 +352,90 @@ def verify_local_collection(
     return result
 
 
+def verify_file_signatures(fqcn, manifest_file, detached_signatures, keyring, required_successful_count, ignore_signature_errors):
+    # type: (str, str, list[str], str, str, list[str]) -> bool
+    successful = 0
+    error_messages = []
+
+    signature_count_requirements = re.match(SIGNATURE_COUNT_RE, required_successful_count).groupdict()
+
+    strict = signature_count_requirements['strict'] or False
+    require_all = signature_count_requirements['all']
+    require_count = signature_count_requirements['count']
+    if require_count is not None:
+        require_count = int(require_count)
+
+    for signature in detached_signatures:
+        signature = to_text(signature, errors='surrogate_or_strict')
+        try:
+            verify_file_signature(manifest_file, signature, keyring, ignore_signature_errors)
+        except CollectionSignatureError as error:
+            if error.ignore:
+                # Do not include ignored errors in either the failed or successful count
+                continue
+            error_messages.append(error.report(fqcn))
+        else:
+            successful += 1
+
+            if require_all:
+                continue
+
+            if successful == require_count:
+                break
+
+    if strict and not successful:
+        verified = False
+        display.display(f"Signature verification failed for '{fqcn}': no successful signatures")
+    elif require_all:
+        verified = not error_messages
+        if not verified:
+            display.display(f"Signature verification failed for '{fqcn}': some signatures failed")
+    else:
+        verified = not detached_signatures or require_count == successful
+        if not verified:
+            display.display(f"Signature verification failed for '{fqcn}': fewer successful signatures than required")
+
+    if not verified:
+        for msg in error_messages:
+            display.vvvv(msg)
+
+    return verified
+
+
+def verify_file_signature(manifest_file, detached_signature, keyring, ignore_signature_errors):
+    # type: (str, str, str, list[str]) -> None
+    """Run the gpg command and parse any errors. Raises CollectionSignatureError on failure."""
+    gpg_result, gpg_verification_rc = run_gpg_verify(manifest_file, detached_signature, keyring, display)
+
+    if gpg_result:
+        errors = parse_gpg_errors(gpg_result)
+        try:
+            error = next(errors)
+        except StopIteration:
+            pass
+        else:
+            reasons = []
+            ignored_reasons = 0
+
+            for error in chain([error], errors):
+                # Get error status (dict key) from the class (dict value)
+                status_code = list(GPG_ERROR_MAP.keys())[list(GPG_ERROR_MAP.values()).index(error.__class__)]
+                if status_code in ignore_signature_errors:
+                    ignored_reasons += 1
+                reasons.append(error.get_gpg_error_description())
+
+            ignore = len(reasons) == ignored_reasons
+            raise CollectionSignatureError(reasons=set(reasons), stdout=gpg_result, rc=gpg_verification_rc, ignore=ignore)
+
+    if gpg_verification_rc:
+        raise CollectionSignatureError(stdout=gpg_result, rc=gpg_verification_rc)
+
+    # No errors and rc is 0, verify was successful
+    return None
+
+
 def build_collection(u_collection_path, u_output_path, force):
-    # type: (Text, Text, bool) -> Text
+    # type: (str, str, bool) -> str
     """Creates the Ansible collection artifact in a .tar.gz file.
 
     :param u_collection_path: The path to the collection to build. This should be the directory that contains the
@@ -292,9 +481,9 @@ def build_collection(u_collection_path, u_output_path, force):
 
 
 def download_collections(
-        collections,  # type: Iterable[Requirement]
+        collections,  # type: t.Iterable[Requirement]
         output_path,  # type: str
-        apis,  # type: Iterable[GalaxyAPI]
+        apis,  # type: t.Iterable[GalaxyAPI]
         no_deps,  # type: bool
         allow_pre_release,  # type: bool
         artifacts_manager,  # type: ConcreteArtifactsManager
@@ -318,6 +507,8 @@ def download_collections(
             no_deps=no_deps,
             allow_pre_release=allow_pre_release,
             upgrade=False,
+            # Avoid overhead getting signatures since they are not currently applicable to downloaded collections
+            include_signatures=False,
         )
 
     b_output_path = to_bytes(output_path, errors='surrogate_or_strict')
@@ -387,7 +578,7 @@ def download_collections(
             "to '{path!s}'".format(path=to_text(requirements_path)),
         )
         yaml_bytes = to_bytes(
-            yaml.safe_dump({'collections': requirements}),
+            yaml_dump({'collections': requirements}),
             errors='surrogate_or_strict',
         )
         with open(b_requirements_path, mode='wb') as req_fd:
@@ -432,9 +623,9 @@ def publish_collection(collection_path, api, wait, timeout):
 
 
 def install_collections(
-        collections,  # type: Iterable[Requirement]
+        collections,  # type: t.Iterable[Requirement]
         output_path,  # type: str
-        apis,  # type: Iterable[GalaxyAPI]
+        apis,  # type: t.Iterable[GalaxyAPI]
         ignore_errors,  # type: bool
         no_deps,  # type: bool
         force,  # type: bool
@@ -442,6 +633,7 @@ def install_collections(
         upgrade,  # type: bool
         allow_pre_release,  # type: bool
         artifacts_manager,  # type: ConcreteArtifactsManager
+        disable_gpg_verify,  # type: bool
 ):  # type: (...) -> None
     """Install Ansible collections to the path specified.
 
@@ -455,7 +647,7 @@ def install_collections(
     :param force_deps: Re-install a collection as well as its dependencies if they have already been installed.
     """
     existing_collections = {
-        Requirement(coll.fqcn, coll.ver, coll.src, coll.type)
+        Requirement(coll.fqcn, coll.ver, coll.src, coll.type, None)
         for coll in find_existing_collections(output_path, artifacts_manager)
     }
 
@@ -505,7 +697,8 @@ def install_collections(
         else existing_collections
     )
     preferred_collections = {
-        Candidate(coll.fqcn, coll.ver, coll.src, coll.type)
+        # NOTE: No need to include signatures if the collection is already installed
+        Candidate(coll.fqcn, coll.ver, coll.src, coll.type, None)
         for coll in preferred_requirements
     }
     with _display_progress("Process install dependency map"):
@@ -517,24 +710,37 @@ def install_collections(
             no_deps=no_deps,
             allow_pre_release=allow_pre_release,
             upgrade=upgrade,
+            include_signatures=not disable_gpg_verify,
         )
 
+    keyring_exists = artifacts_manager.keyring is not None
     with _display_progress("Starting collection install process"):
         for fqcn, concrete_coll_pin in dependency_map.items():
             if concrete_coll_pin.is_virtual:
                 display.vvvv(
-                    "Skipping '{coll!s}' as it is virtual".
+                    "'{coll!s}' is virtual, skipping.".
                     format(coll=to_text(concrete_coll_pin)),
                 )
                 continue
 
             if concrete_coll_pin in preferred_collections:
                 display.display(
-                    "Skipping '{coll!s}' as it is already installed".
+                    "'{coll!s}' is already installed, skipping.".
                     format(coll=to_text(concrete_coll_pin)),
                 )
                 continue
 
+            if not disable_gpg_verify and concrete_coll_pin.signatures and not keyring_exists:
+                # Duplicate warning msgs are not displayed
+                display.warning(
+                    "The GnuPG keyring used for collection signature "
+                    "verification was not configured but signatures were "
+                    "provided by the Galaxy server to verify authenticity. "
+                    "Configure a keyring for ansible-galaxy to use "
+                    "or disable signature verification. "
+                    "Skipping signature verification."
+                )
+
             try:
                 install(concrete_coll_pin, output_path, artifacts_manager)
             except AnsibleError as err:
@@ -583,13 +789,13 @@ def validate_collection_path(collection_path):  # type: (str) -> str
 
 
 def verify_collections(
-        collections,  # type: Iterable[Requirement]
-        search_paths,  # type: Iterable[str]
-        apis,  # type: Iterable[GalaxyAPI]
+        collections,  # type: t.Iterable[Requirement]
+        search_paths,  # type: t.Iterable[str]
+        apis,  # type: t.Iterable[GalaxyAPI]
         ignore_errors,  # type: bool
         local_verify_only,  # type: bool
         artifacts_manager,  # type: ConcreteArtifactsManager
-):  # type: (...) -> List[CollectionVerifyResult]
+):  # type: (...) -> list[CollectionVerifyResult]
     r"""Verify the integrity of locally installed collections.
 
     :param collections: The collections to check.
@@ -600,7 +806,7 @@ def verify_collections(
     :param artifacts_manager: Artifacts manager.
     :return: list of CollectionVerifyResult objects describing the results of each collection verification
     """
-    results = []  # type: List[CollectionVerifyResult]
+    results = []  # type: list[CollectionVerifyResult]
 
     api_proxy = MultiGalaxyAPIProxy(apis, artifacts_manager)
 
@@ -639,6 +845,18 @@ def verify_collections(
                     local_collection = Candidate.from_dir_path(
                         b_search_path, artifacts_manager,
                     )
+                    supplemental_signatures = [
+                        get_signature_from_source(source, display)
+                        for source in collection.signature_sources or []
+                    ]
+                    local_collection = Candidate(
+                        local_collection.fqcn,
+                        local_collection.ver,
+                        local_collection.src,
+                        local_collection.type,
+                        signatures=frozenset(supplemental_signatures),
+                    )
+
                     break
                 else:
                     raise AnsibleError(message=default_err)
@@ -646,20 +864,31 @@ def verify_collections(
                 if local_verify_only:
                     remote_collection = None
                 else:
+                    signatures = api_proxy.get_signatures(local_collection)
+                    signatures.extend([
+                        get_signature_from_source(source, display)
+                        for source in collection.signature_sources or []
+                    ])
+
                     remote_collection = Candidate(
                         collection.fqcn,
                         collection.ver if collection.ver != '*'
                         else local_collection.ver,
                         None, 'galaxy',
+                        frozenset(signatures),
                     )
 
                     # Download collection on a galaxy server for comparison
                     try:
-                        # NOTE: Trigger the lookup. If found, it'll cache
-                        # NOTE: download URL and token in artifact manager.
-                        api_proxy.get_collection_version_metadata(
-                            remote_collection,
-                        )
+                        # NOTE: If there are no signatures, trigger the lookup. If found,
+                        # NOTE: it'll cache download URL and token in artifact manager.
+                        # NOTE: If there are no Galaxy server signatures, only user-provided signature URLs,
+                        # NOTE: those alone validate the MANIFEST.json and the remote collection is not downloaded.
+                        # NOTE: The remote MANIFEST.json is only used in verification if there are no signatures.
+                        if not signatures and not collection.signature_sources:
+                            api_proxy.get_collection_version_metadata(
+                                remote_collection,
+                            )
                     except AnsibleError as e:  # FIXME: does this actually emit any errors?
                         # FIXME: extract the actual message and adjust this:
                         expected_error_msg = (
@@ -786,10 +1015,12 @@ def _verify_file_hash(b_path, filename, expected_hash, error_queue):
 
 
 def _build_files_manifest(b_collection_path, namespace, name, ignore_patterns):
-    # type: (bytes, str, str, List[str]) -> FilesManifestType
+    # type: (bytes, str, str, list[str]) -> FilesManifestType
     # We always ignore .pyc and .retry files as well as some well known version control directories. The ignore
     # patterns can be extended by the build_ignore key in galaxy.yml
     b_ignore_patterns = [
+        b'MANIFEST.json',
+        b'FILES.json',
         b'galaxy.yml',
         b'galaxy.yaml',
         b'.git',
@@ -908,7 +1139,7 @@ def _build_collection_tar(
         b_tar_path,  # type: bytes
         collection_manifest,  # type: CollectionManifestType
         file_manifest,  # type: FilesManifestType
-):  # type: (...) -> Text
+):  # type: (...) -> str
     """Build a tar.gz collection artifact from the manifest data."""
     files_manifest_json = to_bytes(json.dumps(file_manifest, indent=True), errors='surrogate_or_strict')
     collection_manifest['file_manifest_file']['chksum_sha256'] = secure_hash_s(files_manifest_json, hash_func=sha256)
@@ -1076,7 +1307,21 @@ def install(collection, path, artifacts_manager):  # FIXME: mv to dataclasses?
     if collection.is_dir:
         install_src(collection, b_artifact_path, b_collection_path, artifacts_manager)
     else:
-        install_artifact(b_artifact_path, b_collection_path, artifacts_manager._b_working_directory)
+        install_artifact(
+            b_artifact_path,
+            b_collection_path,
+            artifacts_manager._b_working_directory,
+            collection.signatures,
+            artifacts_manager.keyring,
+            artifacts_manager.required_successful_signature_count,
+            artifacts_manager.ignore_signature_errors,
+        )
+        if (collection.is_online_index_pointer and isinstance(collection.src, GalaxyAPI)):
+            write_source_metadata(
+                collection,
+                b_collection_path,
+                artifacts_manager
+            )
 
     display.display(
         '{coll!s} was installed successfully'.
@@ -1084,20 +1329,63 @@ def install(collection, path, artifacts_manager):  # FIXME: mv to dataclasses?
     )
 
 
-def install_artifact(b_coll_targz_path, b_collection_path, b_temp_path):
+def write_source_metadata(collection, b_collection_path, artifacts_manager):
+    # type: (Candidate, bytes, ConcreteArtifactsManager) -> None
+    source_data = artifacts_manager.get_galaxy_artifact_source_info(collection)
+
+    b_yaml_source_data = to_bytes(yaml_dump(source_data), errors='surrogate_or_strict')
+    b_info_dest = collection.construct_galaxy_info_path(b_collection_path)
+    b_info_dir = os.path.split(b_info_dest)[0]
+
+    if os.path.exists(b_info_dir):
+        shutil.rmtree(b_info_dir)
+
+    try:
+        os.mkdir(b_info_dir, mode=0o0755)
+        with open(b_info_dest, mode='w+b') as fd:
+            fd.write(b_yaml_source_data)
+        os.chmod(b_info_dest, 0o0644)
+    except Exception:
+        # Ensure we don't leave the dir behind in case of a failure.
+        if os.path.isdir(b_info_dir):
+            shutil.rmtree(b_info_dir)
+        raise
+
+
+def verify_artifact_manifest(manifest_file, signatures, keyring, required_signature_count, ignore_signature_errors):
+    # type: (str, list[str], str, str, list[str]) -> None
+    failed_verify = False
+    coll_path_parts = to_text(manifest_file, errors='surrogate_or_strict').split(os.path.sep)
+    collection_name = '%s.%s' % (coll_path_parts[-3], coll_path_parts[-2])  # get 'ns' and 'coll' from /path/to/ns/coll/MANIFEST.json
+    if not verify_file_signatures(collection_name, manifest_file, signatures, keyring, required_signature_count, ignore_signature_errors):
+        raise AnsibleError(f"Not installing {collection_name} because GnuPG signature verification failed.")
+    display.vvvv(f"GnuPG signature verification succeeded for {collection_name}")
+
+
+def install_artifact(b_coll_targz_path, b_collection_path, b_temp_path, signatures, keyring, required_signature_count, ignore_signature_errors):
     """Install a collection from tarball under a given path.
 
     :param b_coll_targz_path: Collection tarball to be installed.
     :param b_collection_path: Collection dirs layout path.
     :param b_temp_path: Temporary dir path.
+    :param signatures: frozenset of signatures to verify the MANIFEST.json
+    :param keyring: The keyring used during GPG verification
+    :param required_signature_count: The number of signatures that must successfully verify the collection
+    :param ignore_signature_errors: GPG errors to ignore during signature verification
     """
     try:
         with tarfile.open(b_coll_targz_path, mode='r') as collection_tar:
+            # Verify the signature on the MANIFEST.json before extracting anything else
+            _extract_tar_file(collection_tar, MANIFEST_FILENAME, b_collection_path, b_temp_path)
+
+            if keyring is not None:
+                manifest_file = os.path.join(to_text(b_collection_path, errors='surrogate_or_strict'), MANIFEST_FILENAME)
+                verify_artifact_manifest(manifest_file, signatures, keyring, required_signature_count, ignore_signature_errors)
+
             files_member_obj = collection_tar.getmember('FILES.json')
             with _tarfile_extract(collection_tar, files_member_obj) as (dummy, files_obj):
                 files = json.loads(to_text(files_obj.read(), errors='surrogate_or_strict'))
 
-            _extract_tar_file(collection_tar, MANIFEST_FILENAME, b_collection_path, b_temp_path)
             _extract_tar_file(collection_tar, 'FILES.json', b_collection_path, b_temp_path)
 
             for file_info in files['files']:
@@ -1304,15 +1592,37 @@ def _is_child_path(path, parent_path, link_name=None):
 
 
 def _resolve_depenency_map(
-        requested_requirements,  # type: Iterable[Requirement]
-        galaxy_apis,  # type: Iterable[GalaxyAPI]
+        requested_requirements,  # type: t.Iterable[Requirement]
+        galaxy_apis,  # type: t.Iterable[GalaxyAPI]
         concrete_artifacts_manager,  # type: ConcreteArtifactsManager
-        preferred_candidates,  # type: Optional[Iterable[Candidate]]
+        preferred_candidates,  # type: t.Iterable[Candidate] | None
         no_deps,  # type: bool
         allow_pre_release,  # type: bool
         upgrade,  # type: bool
-):  # type: (...) -> Dict[str, Candidate]
+        include_signatures,  # type: bool
+):  # type: (...) -> dict[str, Candidate]
     """Return the resolved dependency map."""
+    if not HAS_RESOLVELIB:
+        raise AnsibleError("Failed to import resolvelib, check that a supported version is installed")
+    if not HAS_PACKAGING:
+        raise AnsibleError("Failed to import packaging, check that a supported version is installed")
+    try:
+        dist = distribution('ansible-core')
+    except Exception:
+        req = None
+    else:
+        req = next((rr for r in (dist.requires or []) if (rr := PkgReq(r)).name == 'resolvelib'), None)
+    finally:
+        if req is None:
+            # TODO: replace the hardcoded versions with a warning if the dist info is missing
+            # display.warning("Unable to find 'ansible-core' distribution requirements to verify the resolvelib version is supported.")
+            if not RESOLVELIB_LOWERBOUND <= RESOLVELIB_VERSION < RESOLVELIB_UPPERBOUND:
+                raise AnsibleError(
+                    f"ansible-galaxy requires resolvelib<{RESOLVELIB_UPPERBOUND.vstring},>={RESOLVELIB_LOWERBOUND.vstring}"
+                )
+        elif not req.specifier.contains(RESOLVELIB_VERSION.vstring):
+            raise AnsibleError(f"ansible-galaxy requires {req.name}{req.specifier}")
+
     collection_dep_resolver = build_collection_dependency_resolver(
         galaxy_apis=galaxy_apis,
         concrete_artifacts_manager=concrete_artifacts_manager,
@@ -1321,6 +1631,7 @@ def _resolve_depenency_map(
         with_deps=not no_deps,
         with_pre_releases=allow_pre_release,
         upgrade=upgrade,
+        include_signatures=include_signatures,
     )
     try:
         return collection_dep_resolver.resolve(
@@ -1338,14 +1649,14 @@ def _resolve_depenency_map(
             )
             for req_inf in dep_exc.causes
         )
-        error_msg_lines = chain(
+        error_msg_lines = list(chain(
             (
                 'Failed to resolve the requested '
                 'dependencies map. Could not satisfy the following '
                 'requirements:',
             ),
             conflict_causes,
-        )
+        ))
         raise raise_from(  # NOTE: Leading "raise" is a hack for mypy bug #9717
             AnsibleError('\n'.join(error_msg_lines)),
             dep_exc,
@@ -1380,3 +1691,5 @@ def _resolve_depenency_map(
             AnsibleError('\n'.join(error_msg_lines)),
             dep_exc,
         )
+    except ValueError as exc:
+        raise AnsibleError(to_native(exc)) from exc
