@@ -10,24 +10,16 @@ import json
 import os
 import tarfile
 import subprocess
+import typing as t
+
 from contextlib import contextmanager
 from hashlib import sha256
+from urllib.error import URLError
+from urllib.parse import urldefrag
 from shutil import rmtree
 from tempfile import mkdtemp
 
-try:
-    from typing import TYPE_CHECKING
-except ImportError:
-    TYPE_CHECKING = False
-
-if TYPE_CHECKING:
-    from typing import (
-        Any,  # FIXME: !!!111
-        BinaryIO, Dict, IO,
-        Iterator, List, Optional,
-        Set, Tuple, Type, Union,
-    )
-
+if t.TYPE_CHECKING:
     from ansible.galaxy.dependency_resolution.dataclasses import (
         Candidate, Requirement,
     )
@@ -38,8 +30,9 @@ from ansible.galaxy import get_collections_galaxy_meta_info
 from ansible.galaxy.dependency_resolution.dataclasses import _GALAXY_YAML
 from ansible.galaxy.user_agent import user_agent
 from ansible.module_utils._text import to_bytes, to_native, to_text
-from ansible.module_utils.six.moves.urllib.error import URLError
-from ansible.module_utils.six.moves.urllib.parse import urldefrag
+from ansible.module_utils.common.process import get_bin_path
+from ansible.module_utils.common._collections_compat import MutableMapping
+from ansible.module_utils.common.yaml import yaml_load
 from ansible.module_utils.six import raise_from
 from ansible.module_utils.urls import open_url
 from ansible.utils.display import Display
@@ -61,22 +54,77 @@ class ConcreteArtifactsManager:
         * keeping track of local ones
         * keeping track of Galaxy API tokens for downloads from Galaxy'ish
           as well as the artifact hashes
+        * keeping track of Galaxy API signatures for downloads from Galaxy'ish
         * caching all of above
         * retrieving the metadata out of the downloaded artifacts
     """
-
-    def __init__(self, b_working_directory, validate_certs=True):
-        # type: (bytes, bool) -> None
+    def __init__(self, b_working_directory, validate_certs=True, keyring=None, timeout=60, required_signature_count=None, ignore_signature_errors=None):
+        # type: (bytes, bool, str, int, str, list[str]) -> None
         """Initialize ConcreteArtifactsManager caches and costraints."""
         self._validate_certs = validate_certs  # type: bool
-        self._artifact_cache = {}  # type: Dict[bytes, bytes]
-        self._galaxy_artifact_cache = {}  # type: Dict[Union[Candidate, Requirement], bytes]
-        self._artifact_meta_cache = {}  # type: Dict[bytes, Dict[str, Optional[Union[str, List[str], Dict[str, str]]]]]
-        self._galaxy_collection_cache = {}  # type: Dict[Union[Candidate, Requirement], Tuple[str, str, GalaxyToken]]
+        self._artifact_cache = {}  # type: dict[bytes, bytes]
+        self._galaxy_artifact_cache = {}  # type: dict[Candidate | Requirement, bytes]
+        self._artifact_meta_cache = {}  # type: dict[bytes, dict[str, str | list[str] | dict[str, str] | None]]
+        self._galaxy_collection_cache = {}  # type: dict[Candidate | Requirement, tuple[str, str, GalaxyToken]]
+        self._galaxy_collection_origin_cache = {}  # type: dict[Candidate, tuple[str, list[dict[str, str]]]]
         self._b_working_directory = b_working_directory  # type: bytes
+        self._supplemental_signature_cache = {}  # type: dict[str, str]
+        self._keyring = keyring  # type: str
+        self.timeout = timeout  # type: int
+        self._required_signature_count = required_signature_count  # type: str
+        self._ignore_signature_errors = ignore_signature_errors  # type: list[str]
+        self._require_build_metadata = True  # type: bool
+
+    @property
+    def keyring(self):
+        return self._keyring
+
+    @property
+    def required_successful_signature_count(self):
+        return self._required_signature_count
+
+    @property
+    def ignore_signature_errors(self):
+        if self._ignore_signature_errors is None:
+            return []
+        return self._ignore_signature_errors
+
+    @property
+    def require_build_metadata(self):
+        # type: () -> bool
+        return self._require_build_metadata
+
+    @require_build_metadata.setter
+    def require_build_metadata(self, value):
+        # type: (bool) -> None
+        self._require_build_metadata = value
+
+    def get_galaxy_artifact_source_info(self, collection):
+        # type: (Candidate) -> dict[str, t.Union[str, list[dict[str, str]]]]
+        server = collection.src.api_server
+
+        try:
+            download_url = self._galaxy_collection_cache[collection][0]
+            signatures_url, signatures = self._galaxy_collection_origin_cache[collection]
+        except KeyError as key_err:
+            raise RuntimeError(
+                'The is no known source for {coll!s}'.
+                format(coll=collection),
+            ) from key_err
+
+        return {
+            "format_version": "1.0.0",
+            "namespace": collection.namespace,
+            "name": collection.name,
+            "version": collection.ver,
+            "server": server,
+            "version_url": signatures_url,
+            "download_url": download_url,
+            "signatures": signatures,
+        }
 
     def get_galaxy_artifact_path(self, collection):
-        # type: (Union[Candidate, Requirement]) -> bytes
+        # type: (t.Union[Candidate, Requirement]) -> bytes
         """Given a Galaxy-stored collection, return a cached path.
 
         If it's not yet on disk, this method downloads the artifact first.
@@ -136,7 +184,7 @@ class ConcreteArtifactsManager:
         return b_artifact_path
 
     def get_artifact_path(self, collection):
-        # type: (Union[Candidate, Requirement]) -> bytes
+        # type: (t.Union[Candidate, Requirement]) -> bytes
         """Given a concrete collection pointer, return a cached path.
 
         If it's not yet on disk, this method downloads the artifact first.
@@ -168,6 +216,7 @@ class ConcreteArtifactsManager:
                     self._b_working_directory,
                     expected_hash=None,  # NOTE: URLs don't support checksums
                     validate_certs=self._validate_certs,
+                    timeout=self.timeout
                 )
             except URLError as err:
                 raise_from(
@@ -200,15 +249,15 @@ class ConcreteArtifactsManager:
         return b_artifact_path
 
     def _get_direct_collection_namespace(self, collection):
-        # type: (Candidate) -> Optional[str]
+        # type: (Candidate) -> t.Optional[str]
         return self.get_direct_collection_meta(collection)['namespace']  # type: ignore[return-value]
 
     def _get_direct_collection_name(self, collection):
-        # type: (Candidate) -> Optional[str]
+        # type: (Candidate) -> t.Optional[str]
         return self.get_direct_collection_meta(collection)['name']  # type: ignore[return-value]
 
     def get_direct_collection_fqcn(self, collection):
-        # type: (Candidate) -> Optional[str]
+        # type: (Candidate) -> t.Optional[str]
         """Extract FQCN from the given on-disk collection artifact.
 
         If the collection is virtual, ``None`` is returned instead
@@ -224,17 +273,17 @@ class ConcreteArtifactsManager:
         ))
 
     def get_direct_collection_version(self, collection):
-        # type: (Union[Candidate, Requirement]) -> str
+        # type: (t.Union[Candidate, Requirement]) -> str
         """Extract version from the given on-disk collection artifact."""
         return self.get_direct_collection_meta(collection)['version']  # type: ignore[return-value]
 
     def get_direct_collection_dependencies(self, collection):
-        # type: (Union[Candidate, Requirement]) -> Dict[str, str]
+        # type: (t.Union[Candidate, Requirement]) -> dict[str, str]
         """Extract deps from the given on-disk collection artifact."""
         return self.get_direct_collection_meta(collection)['dependencies']  # type: ignore[return-value]
 
     def get_direct_collection_meta(self, collection):
-        # type: (Union[Candidate, Requirement]) -> Dict[str, Optional[Union[str, Dict[str, str], List[str]]]]
+        # type: (t.Union[Candidate, Requirement]) -> dict[str, t.Union[str, dict[str, str], list[str], None]]
         """Extract meta from the given on-disk collection artifact."""
         try:  # FIXME: use unique collection identifier as a cache key?
             return self._artifact_meta_cache[collection.src]
@@ -246,7 +295,7 @@ class ConcreteArtifactsManager:
         elif collection.is_dir:  # should we just build a coll instead?
             # FIXME: what if there's subdirs?
             try:
-                collection_meta = _get_meta_from_dir(b_artifact_path)
+                collection_meta = _get_meta_from_dir(b_artifact_path, self.require_build_metadata)
             except LookupError as lookup_err:
                 raise_from(
                     AnsibleError(
@@ -279,22 +328,27 @@ class ConcreteArtifactsManager:
         self._artifact_meta_cache[collection.src] = collection_meta
         return collection_meta
 
-    def save_collection_source(self, collection, url, sha256_hash, token):
-        # type: (Candidate, str, str, GalaxyToken) -> None
+    def save_collection_source(self, collection, url, sha256_hash, token, signatures_url, signatures):
+        # type: (Candidate, str, str, GalaxyToken, str, list[dict[str, str]]) -> None
         """Store collection URL, SHA256 hash and Galaxy API token.
 
         This is a hook that is supposed to be called before attempting to
         download Galaxy-based collections with ``get_galaxy_artifact_path()``.
         """
         self._galaxy_collection_cache[collection] = url, sha256_hash, token
+        self._galaxy_collection_origin_cache[collection] = signatures_url, signatures
 
     @classmethod
     @contextmanager
     def under_tmpdir(
-            cls,  # type: Type[ConcreteArtifactsManager]
+            cls,
             temp_dir_base,  # type: str
             validate_certs=True,  # type: bool
-    ):  # type: (...) -> Iterator[ConcreteArtifactsManager]
+            keyring=None,  # type: str
+            required_signature_count=None,  # type: str
+            ignore_signature_errors=None,  # type: list[str]
+            require_build_metadata=True,  # type: bool
+    ):  # type: (...) -> t.Iterator[ConcreteArtifactsManager]
         """Custom ConcreteArtifactsManager constructor with temp dir.
 
         This method returns a context manager that allocates and cleans
@@ -308,7 +362,13 @@ class ConcreteArtifactsManager:
         )
         b_temp_path = to_bytes(temp_path, errors='surrogate_or_strict')
         try:
-            yield cls(b_temp_path, validate_certs)
+            yield cls(
+                b_temp_path,
+                validate_certs,
+                keyring=keyring,
+                required_signature_count=required_signature_count,
+                ignore_signature_errors=ignore_signature_errors
+            )
         finally:
             rmtree(b_temp_path)
 
@@ -346,8 +406,22 @@ def _extract_collection_from_git(repo_url, coll_ver, b_path):
         dir=b_path,
         prefix=to_bytes(name, errors='surrogate_or_strict'),
     )  # type: bytes
-    git_clone_cmd = 'git', 'clone', git_url, to_text(b_checkout_path)
-    # FIXME: '--depth', '1', '--branch', version
+
+    try:
+        git_executable = get_bin_path('git')
+    except ValueError as err:
+        raise AnsibleError(
+            "Could not find git executable to extract the collection from the Git repository `{repo_url!s}`.".
+            format(repo_url=to_native(git_url))
+        ) from err
+
+    # Perform a shallow clone if simply cloning HEAD
+    if version == 'HEAD':
+        git_clone_cmd = git_executable, 'clone', '--depth=1', git_url, to_text(b_checkout_path)
+    else:
+        git_clone_cmd = git_executable, 'clone', git_url, to_text(b_checkout_path)
+    # FIXME: '--branch', version
+
     try:
         subprocess.check_call(git_clone_cmd)
     except subprocess.CalledProcessError as proc_err:
@@ -359,7 +433,7 @@ def _extract_collection_from_git(repo_url, coll_ver, b_path):
             proc_err,
         )
 
-    git_switch_cmd = 'git', 'checkout', to_text(version)
+    git_switch_cmd = git_executable, 'checkout', to_text(version)
     try:
         subprocess.check_call(git_switch_cmd, cwd=b_checkout_path)
     except subprocess.CalledProcessError as proc_err:
@@ -382,8 +456,8 @@ def _extract_collection_from_git(repo_url, coll_ver, b_path):
 
 
 # FIXME: use random subdirs while preserving the file names
-def _download_file(url, b_path, expected_hash, validate_certs, token=None):
-    # type: (str, bytes, Optional[str], bool, GalaxyToken) -> bytes
+def _download_file(url, b_path, expected_hash, validate_certs, token=None, timeout=60):
+    # type: (str, bytes, t.Optional[str], bool, GalaxyToken, int) -> bytes
     # ^ NOTE: used in download and verify_collections ^
     b_tarball_name = to_bytes(
         url.rsplit('/', 1)[1], errors='surrogate_or_strict',
@@ -405,9 +479,10 @@ def _download_file(url, b_path, expected_hash, validate_certs, token=None):
         validate_certs=validate_certs,
         headers=None if token is None else token.headers(),
         unredirected_headers=['Authorization'], http_agent=user_agent(),
+        timeout=timeout
     )
 
-    with open(b_file_path, 'wb') as download_file:  # type: BinaryIO
+    with open(b_file_path, 'wb') as download_file:  # type: t.BinaryIO
         actual_hash = _consume_file(resp, write_to=download_file)
 
     if expected_hash:
@@ -423,7 +498,7 @@ def _download_file(url, b_path, expected_hash, validate_certs, token=None):
 
 
 def _consume_file(read_from, write_to=None):
-    # type: (BinaryIO, BinaryIO) -> str
+    # type: (t.BinaryIO, t.BinaryIO) -> str
     bufsize = 65536
     sha256_digest = sha256()
     data = read_from.read(bufsize)
@@ -438,19 +513,20 @@ def _consume_file(read_from, write_to=None):
 
 
 def _normalize_galaxy_yml_manifest(
-        galaxy_yml,  # type: Dict[str, Optional[Union[str, List[str], Dict[str, str]]]]
+        galaxy_yml,  # type: dict[str, t.Union[str, list[str], dict[str, str], None]]
         b_galaxy_yml_path,  # type: bytes
+        require_build_metadata=True,  # type: bool
 ):
-    # type: (...) -> Dict[str, Optional[Union[str, List[str], Dict[str, str]]]]
+    # type: (...) -> dict[str, t.Union[str, list[str], dict[str, str], None]]
     galaxy_yml_schema = (
         get_collections_galaxy_meta_info()
-    )  # type: List[Dict[str, Any]]  # FIXME: <--
-    # FIXME: 👆maybe precise type: List[Dict[str, Union[bool, str, List[str]]]]
+    )  # type: list[dict[str, t.Any]]  # FIXME: <--
+    # FIXME: 👆maybe precise type: list[dict[str, t.Union[bool, str, list[str]]]]
 
     mandatory_keys = set()
-    string_keys = set()  # type: Set[str]
-    list_keys = set()  # type: Set[str]
-    dict_keys = set()  # type: Set[str]
+    string_keys = set()  # type: set[str]
+    list_keys = set()  # type: set[str]
+    dict_keys = set()  # type: set[str]
 
     for info in galaxy_yml_schema:
         if info.get('required', False):
@@ -468,8 +544,14 @@ def _normalize_galaxy_yml_manifest(
     set_keys = set(galaxy_yml.keys())
     missing_keys = mandatory_keys.difference(set_keys)
     if missing_keys:
-        raise AnsibleError("The collection galaxy.yml at '%s' is missing the following mandatory keys: %s"
-                           % (to_native(b_galaxy_yml_path), ", ".join(sorted(missing_keys))))
+        msg = (
+            "The collection galaxy.yml at '%s' is missing the following mandatory keys: %s"
+            % (to_native(b_galaxy_yml_path), ", ".join(sorted(missing_keys)))
+        )
+        if require_build_metadata:
+            raise AnsibleError(msg)
+        display.warning(msg)
+        raise ValueError(msg)
 
     extra_keys = set_keys.difference(all_keys)
     if len(extra_keys) > 0:
@@ -505,16 +587,18 @@ def _normalize_galaxy_yml_manifest(
 
 def _get_meta_from_dir(
         b_path,  # type: bytes
-):  # type: (...) -> Dict[str, Optional[Union[str, List[str], Dict[str, str]]]]
+        require_build_metadata=True,  # type: bool
+):  # type: (...) -> dict[str, t.Union[str, list[str], dict[str, str], None]]
     try:
         return _get_meta_from_installed_dir(b_path)
     except LookupError:
-        return _get_meta_from_src_dir(b_path)
+        return _get_meta_from_src_dir(b_path, require_build_metadata)
 
 
 def _get_meta_from_src_dir(
         b_path,  # type: bytes
-):  # type: (...) -> Dict[str, Optional[Union[str, List[str], Dict[str, str]]]]
+        require_build_metadata=True,  # type: bool
+):  # type: (...) -> dict[str, t.Union[str, list[str], dict[str, str], None]]
     galaxy_yml = os.path.join(b_path, _GALAXY_YAML)
     if not os.path.isfile(galaxy_yml):
         raise LookupError(
@@ -524,7 +608,7 @@ def _get_meta_from_src_dir(
 
     with open(galaxy_yml, 'rb') as manifest_file_obj:
         try:
-            manifest = yaml.safe_load(manifest_file_obj)
+            manifest = yaml_load(manifest_file_obj)
         except yaml.error.YAMLError as yaml_err:
             raise_from(
                 AnsibleError(
@@ -538,13 +622,20 @@ def _get_meta_from_src_dir(
                 yaml_err,
             )
 
-    return _normalize_galaxy_yml_manifest(manifest, galaxy_yml)
+    if not isinstance(manifest, dict):
+        if require_build_metadata:
+            raise AnsibleError(f"The collection galaxy.yml at '{to_native(galaxy_yml)}' is incorrectly formatted.")
+        # Valid build metadata is not required by ansible-galaxy list. Raise ValueError to fall back to implicit metadata.
+        display.warning(f"The collection galaxy.yml at '{to_native(galaxy_yml)}' is incorrectly formatted.")
+        raise ValueError(f"The collection galaxy.yml at '{to_native(galaxy_yml)}' is incorrectly formatted.")
+
+    return _normalize_galaxy_yml_manifest(manifest, galaxy_yml, require_build_metadata)
 
 
 def _get_json_from_installed_dir(
         b_path,  # type: bytes
         filename,  # type: str
-):  # type: (...) -> Dict
+):  # type: (...) -> dict
 
     b_json_filepath = os.path.join(b_path, to_bytes(filename, errors='surrogate_or_strict'))
 
@@ -576,7 +667,7 @@ def _get_json_from_installed_dir(
 
 def _get_meta_from_installed_dir(
         b_path,  # type: bytes
-):  # type: (...) -> Dict[str, Optional[Union[str, List[str], Dict[str, str]]]]
+):  # type: (...) -> dict[str, t.Union[str, list[str], dict[str, str], None]]
     manifest = _get_json_from_installed_dir(b_path, MANIFEST_FILENAME)
     collection_info = manifest['collection_info']
 
@@ -597,7 +688,7 @@ def _get_meta_from_installed_dir(
 
 def _get_meta_from_tar(
         b_path,  # type: bytes
-):  # type: (...) -> Dict[str, Optional[Union[str, List[str], Dict[str, str]]]]
+):  # type: (...) -> dict[str, t.Union[str, list[str], dict[str, str], None]]
     if not tarfile.is_tarfile(b_path):
         raise AnsibleError(
             "Collection artifact at '{path!s}' is not a valid tar file.".
@@ -645,7 +736,7 @@ def _tarfile_extract(
         tar,  # type: tarfile.TarFile
         member,  # type: tarfile.TarInfo
 ):
-    # type: (...) -> Iterator[Tuple[tarfile.TarInfo, Optional[IO[bytes]]]]
+    # type: (...) -> t.Iterator[tuple[tarfile.TarInfo, t.Optional[t.IO[bytes]]]]
     tar_obj = tar.extractfile(member)
     try:
         yield member, tar_obj
