@@ -52,6 +52,7 @@ import socket
 import sys
 import tempfile
 import traceback
+import types
 
 from contextlib import contextmanager
 
@@ -65,14 +66,14 @@ try:
     import httplib
 except ImportError:
     # Python 3
-    import http.client as httplib
+    import http.client as httplib  # type: ignore[no-redef]
 
+import ansible.module_utils.compat.typing as t
 import ansible.module_utils.six.moves.http_cookiejar as cookiejar
-import ansible.module_utils.six.moves.urllib.request as urllib_request
 import ansible.module_utils.six.moves.urllib.error as urllib_error
 
 from ansible.module_utils.common.collections import Mapping
-from ansible.module_utils.six import PY3, string_types
+from ansible.module_utils.six import PY2, PY3, string_types
 from ansible.module_utils.six.moves import cStringIO
 from ansible.module_utils.basic import get_distribution, missing_required_lib
 from ansible.module_utils._text import to_bytes, to_native, to_text
@@ -83,13 +84,13 @@ try:
     from urllib.request import AbstractHTTPHandler, BaseHandler
 except ImportError:
     # python2
-    import urllib2 as urllib_request
-    from urllib2 import AbstractHTTPHandler, BaseHandler
+    import urllib2 as urllib_request  # type: ignore[no-redef]
+    from urllib2 import AbstractHTTPHandler, BaseHandler  # type: ignore[no-redef]
 
-urllib_request.HTTPRedirectHandler.http_error_308 = urllib_request.HTTPRedirectHandler.http_error_307
+urllib_request.HTTPRedirectHandler.http_error_308 = urllib_request.HTTPRedirectHandler.http_error_307  # type: ignore[attr-defined]
 
 try:
-    from ansible.module_utils.six.moves.urllib.parse import urlparse, urlunparse
+    from ansible.module_utils.six.moves.urllib.parse import urlparse, urlunparse, unquote
     HAS_URLPARSE = True
 except Exception:
     HAS_URLPARSE = False
@@ -113,19 +114,19 @@ try:
     HAS_URLLIB3_SSL_WRAP_SOCKET = False
     try:
         from urllib3.contrib.pyopenssl import PyOpenSSLContext
-    except ImportError:
+    except Exception:
         from requests.packages.urllib3.contrib.pyopenssl import PyOpenSSLContext
     HAS_URLLIB3_PYOPENSSLCONTEXT = True
-except ImportError:
+except Exception:
     # urllib3<1.15,>=1.6
     HAS_URLLIB3_PYOPENSSLCONTEXT = False
     try:
         try:
             from urllib3.contrib.pyopenssl import ssl_wrap_socket
-        except ImportError:
+        except Exception:
             from requests.packages.urllib3.contrib.pyopenssl import ssl_wrap_socket
         HAS_URLLIB3_SSL_WRAP_SOCKET = True
-    except ImportError:
+    except Exception:
         pass
 
 # Select a protocol that includes all secure tls protocols
@@ -160,14 +161,14 @@ if not HAS_SSLCONTEXT and HAS_SSL:
 # The bundled backports.ssl_match_hostname should really be moved into its own file for processing
 _BUNDLED_METADATA = {"pypi_name": "backports.ssl_match_hostname", "version": "3.7.0.1"}
 
-LOADED_VERIFY_LOCATIONS = set()
+LOADED_VERIFY_LOCATIONS = set()  # type: t.Set[str]
 
 HAS_MATCH_HOSTNAME = True
 try:
     from ssl import match_hostname, CertificateError
 except ImportError:
     try:
-        from backports.ssl_match_hostname import match_hostname, CertificateError
+        from backports.ssl_match_hostname import match_hostname, CertificateError  # type: ignore[misc]
     except ImportError:
         HAS_MATCH_HOSTNAME = False
 
@@ -268,7 +269,7 @@ try:
 
 except ImportError:
     GSSAPI_IMP_ERR = traceback.format_exc()
-    HTTPGSSAPIAuthHandler = None
+    HTTPGSSAPIAuthHandler = None  # type: types.ModuleType | None  # type: ignore[no-redef]
 
 if not HAS_MATCH_HOSTNAME:
     # The following block of code is under the terms and conditions of the
@@ -279,9 +280,9 @@ if not HAS_MATCH_HOSTNAME:
     try:
         # Divergence: Python-3.7+'s _ssl has this exception type but older Pythons do not
         from _ssl import SSLCertVerificationError
-        CertificateError = SSLCertVerificationError
+        CertificateError = SSLCertVerificationError  # type: ignore[misc]
     except ImportError:
-        class CertificateError(ValueError):
+        class CertificateError(ValueError):  # type: ignore[no-redef]
             pass
 
     def _dnsname_match(dn, hostname):
@@ -390,7 +391,7 @@ if not HAS_MATCH_HOSTNAME:
         ip = _inet_paton(ipname.rstrip())
         return ip == host_ip
 
-    def match_hostname(cert, hostname):
+    def match_hostname(cert, hostname):  # type: ignore[misc]
         """Verify that *cert* (in decoded format as returned by
         SSLSocket.getpeercert()) matches the *hostname*.  RFC 2818 and RFC 6125
         rules are followed.
@@ -475,6 +476,11 @@ zKPZsZ2miVGclicJHzm5q080b1p/sZtuKIEZk6vZqEg=
 -----END CERTIFICATE-----
 """
 
+b_PEM_CERT_RE = re.compile(
+    br'^-----BEGIN CERTIFICATE-----\n.+?-----END CERTIFICATE-----$',
+    flags=re.M | re.S
+)
+
 #
 # Exceptions
 #
@@ -514,7 +520,7 @@ CustomHTTPSHandler = None
 HTTPSClientAuthHandler = None
 UnixHTTPSConnection = None
 if hasattr(httplib, 'HTTPSConnection') and hasattr(urllib_request, 'HTTPSHandler'):
-    class CustomHTTPSConnection(httplib.HTTPSConnection):
+    class CustomHTTPSConnection(httplib.HTTPSConnection):  # type: ignore[no-redef]
         def __init__(self, *args, **kwargs):
             httplib.HTTPSConnection.__init__(self, *args, **kwargs)
             self.context = None
@@ -549,7 +555,7 @@ if hasattr(httplib, 'HTTPSConnection') and hasattr(urllib_request, 'HTTPSHandler
             else:
                 self.sock = ssl.wrap_socket(sock, keyfile=self.key_file, certfile=self.cert_file, ssl_version=PROTOCOL)
 
-    class CustomHTTPSHandler(urllib_request.HTTPSHandler):
+    class CustomHTTPSHandler(urllib_request.HTTPSHandler):  # type: ignore[no-redef]
 
         def https_open(self, req):
             kwargs = {}
@@ -565,7 +571,7 @@ if hasattr(httplib, 'HTTPSConnection') and hasattr(urllib_request, 'HTTPSHandler
 
         https_request = AbstractHTTPHandler.do_request_
 
-    class HTTPSClientAuthHandler(urllib_request.HTTPSHandler):
+    class HTTPSClientAuthHandler(urllib_request.HTTPSHandler):  # type: ignore[no-redef]
         '''Handles client authentication via cert/key
 
         This is a fairly lightweight extension on HTTPSHandler, and can be used
@@ -605,7 +611,7 @@ if hasattr(httplib, 'HTTPSConnection') and hasattr(urllib_request, 'HTTPSHandler
         yield
         httplib.HTTPConnection.connect = _connect
 
-    class UnixHTTPSConnection(httplib.HTTPSConnection):
+    class UnixHTTPSConnection(httplib.HTTPSConnection):  # type: ignore[no-redef]
         def __init__(self, unix_socket):
             self._unix_socket = unix_socket
 
@@ -745,6 +751,35 @@ def generic_urlparse(parts):
     return generic_parts
 
 
+def extract_pem_certs(b_data):
+    for match in b_PEM_CERT_RE.finditer(b_data):
+        yield match.group(0)
+
+
+def get_response_filename(response):
+    url = response.geturl()
+    path = urlparse(url)[2]
+    filename = os.path.basename(path.rstrip('/')) or None
+    if filename:
+        filename = unquote(filename)
+
+    return response.headers.get_param('filename', header='content-disposition') or filename
+
+
+def parse_content_type(response):
+    if PY2:
+        get_type = response.headers.gettype
+        get_param = response.headers.getparam
+    else:
+        get_type = response.headers.get_content_type
+        get_param = response.headers.get_param
+
+    content_type = (get_type() or 'application/octet-stream').split(',')[0]
+    main_type, sub_type = content_type.split('/')
+    charset = (get_param('charset') or 'utf-8').split(',')[0]
+    return content_type, main_type, sub_type, charset
+
+
 class RequestWithMethod(urllib_request.Request):
     '''
     Workaround for using DELETE/PUT/etc with urllib2
@@ -816,7 +851,7 @@ def RedirectHandlerFactory(follow_redirects=None, validate_certs=True, ca_path=N
             # Be conciliant with URIs containing a space
             newurl = newurl.replace(' ', '%20')
 
-            # Suport redirect with payload and original headers
+            # Support redirect with payload and original headers
             if code in (307, 308):
                 # Preserve payload and headers
                 headers = req.headers
@@ -918,11 +953,12 @@ class SSLValidationHandler(urllib_request.BaseHandler):
             paths_checked = [self.ca_path]
             with open(to_bytes(self.ca_path, errors='surrogate_or_strict'), 'rb') as f:
                 if HAS_SSLCONTEXT:
-                    cadata.extend(
-                        ssl.PEM_cert_to_DER_cert(
-                            to_native(f.read(), errors='surrogate_or_strict')
+                    for b_pem in extract_pem_certs(f.read()):
+                        cadata.extend(
+                            ssl.PEM_cert_to_DER_cert(
+                                to_native(b_pem, errors='surrogate_or_strict')
+                            )
                         )
-                    )
             return self.ca_path, cadata, paths_checked
 
         if not HAS_SSLCONTEXT:
@@ -943,6 +979,9 @@ class SSLValidationHandler(urllib_request.BaseHandler):
             ca_certs.append('/etc/openssl/certs')
         elif system == u'SunOS':
             paths_checked.append('/opt/local/etc/openssl/certs')
+        elif system == u'AIX':
+            paths_checked.append('/var/ssl/certs')
+            paths_checked.append('/opt/freeware/etc/ssl/certs')
 
         # fall back to a user-deployed cert in a standard
         # location if the OS platform one is not available
@@ -981,11 +1020,12 @@ class SSLValidationHandler(urllib_request.BaseHandler):
                                     b_cert = cert_file.read()
                                 if HAS_SSLCONTEXT:
                                     try:
-                                        cadata.extend(
-                                            ssl.PEM_cert_to_DER_cert(
-                                                to_native(b_cert, errors='surrogate_or_strict')
+                                        for b_pem in extract_pem_certs(b_cert):
+                                            cadata.extend(
+                                                ssl.PEM_cert_to_DER_cert(
+                                                    to_native(b_pem, errors='surrogate_or_strict')
+                                                )
                                             )
-                                        )
                                     except Exception:
                                         continue
                                 else:
@@ -1436,9 +1476,9 @@ class Request:
             request.add_header('If-Modified-Since', tstamp)
 
         # user defined headers now, which may override things we've set above
-        unredirected_headers = unredirected_headers or []
+        unredirected_headers = [h.lower() for h in (unredirected_headers or [])]
         for header in headers:
-            if header in unredirected_headers:
+            if header.lower() in unredirected_headers:
                 request.add_unredirected_header(header, headers[header])
             else:
                 request.add_header(header, headers[header])
@@ -1673,8 +1713,7 @@ def url_argument_spec():
     '''
     return dict(
         url=dict(type='str'),
-        force=dict(type='bool', default=False, aliases=['thirsty'],
-                   deprecated_aliases=[dict(name='thirsty', version='2.13', collection_name='ansible.builtin')]),
+        force=dict(type='bool', default=False),
         http_agent=dict(type='str', default='ansible-httpget'),
         use_proxy=dict(type='bool', default=True),
         validate_certs=dict(type='bool', default=True),
@@ -1688,8 +1727,8 @@ def url_argument_spec():
 
 
 def fetch_url(module, url, data=None, headers=None, method=None,
-              use_proxy=True, force=False, last_mod_time=None, timeout=10,
-              use_gssapi=False, unix_socket=None, ca_path=None, cookies=None):
+              use_proxy=None, force=False, last_mod_time=None, timeout=10,
+              use_gssapi=False, unix_socket=None, ca_path=None, cookies=None, unredirected_headers=None):
     """Sends a request via HTTP(S) or FTP (needs the module as parameter)
 
     :arg module: The AnsibleModule (used to get username, password etc. (s.b.).
@@ -1698,7 +1737,7 @@ def fetch_url(module, url, data=None, headers=None, method=None,
     :kwarg data:          The data to be sent (in case of POST/PUT).
     :kwarg headers:       A dict with the request headers.
     :kwarg method:        "POST", "PUT", etc.
-    :kwarg boolean use_proxy:     Default: True
+    :kwarg use_proxy:     (optional) whether or not to use proxy (Default: True)
     :kwarg boolean force: If True: Do not get a cached copy (Default: False)
     :kwarg last_mod_time: Default: None
     :kwarg int timeout:   Default: 10
@@ -1706,6 +1745,8 @@ def fetch_url(module, url, data=None, headers=None, method=None,
     :kwarg unix_socket: (optional) String of file system path to unix socket file to use when establishing
         connection to the provided url
     :kwarg ca_path: (optional) String of file system path to CA cert bundle to use
+    :kwarg cookies: (optional) CookieJar object to send with the request
+    :kwarg unredirected_headers: (optional) A list of headers to not attach on a redirected request
 
     :returns: A tuple of (**response**, **info**). Use ``response.read()`` to read the data.
         The **info** contains the 'status' and other meta data. When a HttpError (status >= 400)
@@ -1735,6 +1776,9 @@ def fetch_url(module, url, data=None, headers=None, method=None,
     # Get validate_certs from the module params
     validate_certs = module.params.get('validate_certs', True)
 
+    if use_proxy is None:
+        use_proxy = module.params.get('use_proxy', True)
+
     username = module.params.get('url_username', '')
     password = module.params.get('url_password', '')
     http_agent = module.params.get('http_agent', 'ansible-httpget')
@@ -1758,7 +1802,7 @@ def fetch_url(module, url, data=None, headers=None, method=None,
                      url_password=password, http_agent=http_agent, force_basic_auth=force_basic_auth,
                      follow_redirects=follow_redirects, client_cert=client_cert,
                      client_key=client_key, cookies=cookies, use_gssapi=use_gssapi,
-                     unix_socket=unix_socket, ca_path=ca_path)
+                     unix_socket=unix_socket, ca_path=ca_path, unredirected_headers=unredirected_headers)
         # Lowercase keys, to conform to py2 behavior, so that py3 and py2 are predictable
         info.update(dict((k.lower(), v) for k, v in r.info().items()))
 
@@ -1800,10 +1844,18 @@ def fetch_url(module, url, data=None, headers=None, method=None,
     except MissingModuleError as e:
         module.fail_json(msg=to_text(e), exception=e.import_traceback)
     except urllib_error.HTTPError as e:
+        r = e
         try:
+            if e.fp is None:
+                # Certain HTTPError objects may not have the ability to call ``.read()`` on Python 3
+                # This is not handled gracefully in Python 3, and instead an exception is raised from
+                # tempfile, due to ``urllib.response.addinfourl`` not being initialized
+                raise AttributeError
             body = e.read()
         except AttributeError:
             body = ''
+        else:
+            e.close()
 
         # Try to add exception info to the output but don't fail if we can't
         try:
@@ -1831,7 +1883,8 @@ def fetch_url(module, url, data=None, headers=None, method=None,
 
 
 def fetch_file(module, url, data=None, headers=None, method=None,
-               use_proxy=True, force=False, last_mod_time=None, timeout=10):
+               use_proxy=True, force=False, last_mod_time=None, timeout=10,
+               unredirected_headers=None):
     '''Download and save a file via HTTP(S) or FTP (needs the module as parameter).
     This is basically a wrapper around fetch_url().
 
@@ -1845,6 +1898,7 @@ def fetch_file(module, url, data=None, headers=None, method=None,
     :kwarg boolean force: If True: Do not get a cached copy (Default: False)
     :kwarg last_mod_time: Default: None
     :kwarg int timeout:   Default: 10
+    :kwarg unredirected_headers: (optional) A list of headers to not attach on a redirected request
 
     :returns: A string, the path to the downloaded file.
     '''
@@ -1854,7 +1908,8 @@ def fetch_file(module, url, data=None, headers=None, method=None,
     fetch_temp_file = tempfile.NamedTemporaryFile(dir=module.tmpdir, prefix=file_name, suffix=file_ext, delete=False)
     module.add_cleanup_file(fetch_temp_file.name)
     try:
-        rsp, info = fetch_url(module, url, data, headers, method, use_proxy, force, last_mod_time, timeout)
+        rsp, info = fetch_url(module, url, data, headers, method, use_proxy, force, last_mod_time, timeout,
+                              unredirected_headers=unredirected_headers)
         if not rsp:
             module.fail_json(msg="Failure downloading %s, %s" % (url, info['msg']))
         data = rsp.read(bufsize)
