@@ -1,4 +1,3 @@
-#!/usr/bin/python
 # -*- coding: utf-8 -*-
 
 # Copyright 2015 Cristian van Ee <cristian at cvee.org>
@@ -25,7 +24,10 @@ options:
         When using state=latest, this can be '*' which means run: dnf -y update.
         You can also pass a url or a local path to a rpm file.
         To operate on several packages this can accept a comma separated string of packages or a list of packages."
-      - Comparison operators for package version are valid here C(>), C(<), C(>=), C(<=). Example - C(name>=1.0)
+      - Comparison operators for package version are valid here C(>), C(<), C(>=), C(<=). Example - C(name >= 1.0).
+        Spaces around the operator are required.
+      - You can also pass an absolute path for a binary which is provided by the package to install.
+        See examples for more information.
     required: true
     aliases:
         - pkg
@@ -34,7 +36,8 @@ options:
 
   list:
     description:
-      - Various (non-idempotent) commands for usage with C(/usr/bin/ansible) and I(not) playbooks. See examples.
+      - Various (non-idempotent) commands for usage with C(/usr/bin/ansible) and I(not) playbooks.
+        Use M(ansible.builtin.package_facts) instead of the C(list) argument as a best practice.
     type: str
 
   state:
@@ -107,7 +110,8 @@ options:
     elements: str
   skip_broken:
     description:
-      - Skip packages with broken dependencies(devsolve) and are causing problems.
+      - Skip all unavailable packages or packages with broken dependencies
+        without raising an error. Equivalent to passing the --skip-broken option.
     type: bool
     default: "no"
     version_added: "2.7"
@@ -129,14 +133,14 @@ options:
   security:
     description:
       - If set to C(yes), and C(state=latest) then only installs updates that have been marked security related.
-      - Note that, similar to ``dnf upgrade-minimal``, this filter applies to dependencies as well.
+      - Note that, similar to C(dnf upgrade-minimal), this filter applies to dependencies as well.
     type: bool
     default: "no"
     version_added: "2.7"
   bugfix:
     description:
       - If set to C(yes), and C(state=latest) then only installs updates that have been marked bugfix related.
-      - Note that, similar to ``dnf upgrade-minimal``, this filter applies to dependencies as well.
+      - Note that, similar to C(dnf upgrade-minimal), this filter applies to dependencies as well.
     default: "no"
     type: bool
     version_added: "2.7"
@@ -169,6 +173,13 @@ options:
     type: bool
     default: "yes"
     version_added: "2.7"
+  sslverify:
+    description:
+      - Disables SSL validation of the repository server for this transaction.
+      - This should be set to C(no) if one of the configured repositories is using an untrusted or self-signed certificate.
+    type: bool
+    default: "yes"
+    version_added: "2.13"
   allow_downgrade:
     description:
       - Specify if the named package and version is allowed to downgrade
@@ -228,8 +239,31 @@ options:
     type: bool
     default: "no"
     version_added: "2.11"
+  cacheonly:
+    description:
+      - Tells dnf to run entirely from system cache; does not download or update metadata.
+    type: bool
+    default: "no"
+    version_added: "2.12"
+extends_documentation_fragment:
+- action_common_attributes
+- action_common_attributes.flow
+attributes:
+    action:
+        details: In the case of dnf, it has 2 action plugins that use it under the hood, M(ansible.builtin.yum) and M(ansible.builtin.package).
+        support: partial
+    async:
+        support: none
+    bypass_host_loop:
+        support: none
+    check_mode:
+        support: full
+    diff_mode:
+        support: full
+    platform:
+        platforms: rhel
 notes:
-  - When used with a `loop:` each package will be processed individually, it is much more efficient to pass the list directly to the `name` option.
+  - When used with a C(loop:) each package will be processed individually, it is much more efficient to pass the list directly to the I(name) option.
   - Group removal doesn't work if the group was installed with Ansible because
     upstream dnf's API doesn't properly mark groups as installed, therefore upon
     removal the module is unable to detect that the group is installed
@@ -247,75 +281,80 @@ author:
 
 EXAMPLES = '''
 - name: Install the latest version of Apache
-  dnf:
+  ansible.builtin.dnf:
     name: httpd
     state: latest
 
 - name: Install Apache >= 2.4
-  dnf:
-    name: httpd>=2.4
+  ansible.builtin.dnf:
+    name: httpd >= 2.4
     state: present
 
 - name: Install the latest version of Apache and MariaDB
-  dnf:
+  ansible.builtin.dnf:
     name:
       - httpd
       - mariadb-server
     state: latest
 
 - name: Remove the Apache package
-  dnf:
+  ansible.builtin.dnf:
     name: httpd
     state: absent
 
 - name: Install the latest version of Apache from the testing repo
-  dnf:
+  ansible.builtin.dnf:
     name: httpd
     enablerepo: testing
     state: present
 
 - name: Upgrade all packages
-  dnf:
+  ansible.builtin.dnf:
     name: "*"
     state: latest
 
 - name: Install the nginx rpm from a remote repo
-  dnf:
+  ansible.builtin.dnf:
     name: 'http://nginx.org/packages/centos/6/noarch/RPMS/nginx-release-centos-6-0.el6.ngx.noarch.rpm'
     state: present
 
 - name: Install nginx rpm from a local file
-  dnf:
+  ansible.builtin.dnf:
     name: /usr/local/src/nginx-release-centos-6-0.el6.ngx.noarch.rpm
     state: present
 
+- name: Install Package based upon the file it provides
+  ansible.builtin.dnf:
+    name: /usr/bin/cowsay
+    state: present
+
 - name: Install the 'Development tools' package group
-  dnf:
+  ansible.builtin.dnf:
     name: '@Development tools'
     state: present
 
 - name: Autoremove unneeded packages installed as dependencies
-  dnf:
+  ansible.builtin.dnf:
     autoremove: yes
 
 - name: Uninstall httpd but keep its dependencies
-  dnf:
+  ansible.builtin.dnf:
     name: httpd
     state: absent
     autoremove: no
 
 - name: Install a modularity appstream with defined stream and profile
-  dnf:
+  ansible.builtin.dnf:
     name: '@postgresql:9.6/client'
     state: present
 
 - name: Install a modularity appstream with defined stream
-  dnf:
+  ansible.builtin.dnf:
     name: '@postgresql:9.6'
     state: present
 
 - name: Install a modularity appstream with defined profile
-  dnf:
+  ansible.builtin.dnf:
     name: '@postgresql/client'
     state: present
 '''
@@ -327,22 +366,19 @@ import sys
 from ansible.module_utils._text import to_native, to_text
 from ansible.module_utils.urls import fetch_file
 from ansible.module_utils.six import PY2, text_type
-from distutils.version import LooseVersion
+from ansible.module_utils.compat.version import LooseVersion
 
 from ansible.module_utils.basic import AnsibleModule
+from ansible.module_utils.common.locale import get_best_parsable_locale
 from ansible.module_utils.common.respawn import has_respawned, probe_interpreters_for_module, respawn_module
 from ansible.module_utils.yumdnf import YumDnf, yumdnf_argument_spec
 
-try:
-    import dnf
-    import dnf.cli
-    import dnf.const
-    import dnf.exceptions
-    import dnf.subject
-    import dnf.util
-    HAS_DNF = True
-except ImportError:
-    HAS_DNF = False
+
+# NOTE dnf Python bindings import is postponed, see DnfModule._ensure_dnf(),
+# because we need AnsibleModule object to use get_best_parsable_locale()
+# to set proper locale before importing dnf to be able to scrape
+# the output in some cases (FIXME?).
+dnf = None
 
 
 class DnfModule(YumDnf):
@@ -412,8 +448,13 @@ class DnfModule(YumDnf):
             'release': package.release,
             'version': package.version,
             'repo': package.repoid}
-        result['nevra'] = '{epoch}:{name}-{version}-{release}.{arch}'.format(
-            **result)
+
+        # envra format for alignment with the yum module
+        result['envra'] = '{epoch}:{name}-{version}-{release}.{arch}'.format(**result)
+
+        # keep nevra key for backwards compat as it was previously
+        # defined with a value in envra format
+        result['nevra'] = result['envra']
 
         if package.installtime == 0:
             result['yumstate'] = 'available'
@@ -513,6 +554,22 @@ class DnfModule(YumDnf):
         return rc
 
     def _ensure_dnf(self):
+        locale = get_best_parsable_locale(self.module)
+        os.environ['LC_ALL'] = os.environ['LC_MESSAGES'] = locale
+        os.environ['LANGUAGE'] = os.environ['LANG'] = locale
+
+        global dnf
+        try:
+            import dnf
+            import dnf.cli
+            import dnf.const
+            import dnf.exceptions
+            import dnf.subject
+            import dnf.util
+            HAS_DNF = True
+        except ImportError:
+            HAS_DNF = False
+
         if HAS_DNF:
             return
 
@@ -539,7 +596,7 @@ class DnfModule(YumDnf):
             results=[]
         )
 
-    def _configure_base(self, base, conf_file, disable_gpg_check, installroot='/'):
+    def _configure_base(self, base, conf_file, disable_gpg_check, installroot='/', sslverify=True):
         """Configure the dnf Base object."""
 
         conf = base.conf
@@ -568,6 +625,9 @@ class DnfModule(YumDnf):
         # Don't prompt for user confirmations
         conf.assumeyes = True
 
+        # Set certificate validation
+        conf.sslverify = sslverify
+
         # Set installroot
         conf.installroot = installroot
 
@@ -599,6 +659,14 @@ class DnfModule(YumDnf):
         if self.releasever is not None:
             conf.substitutions['releasever'] = self.releasever
 
+        if conf.substitutions.get('releasever') is None:
+            self.module.warn(
+                'Unable to detect release version (use "releasever" option to specify release version)'
+            )
+            # values of conf.substitutions are expected to be strings
+            # setting this to an empty string instead of None appears to mimic the DNF CLI behavior
+            conf.substitutions['releasever'] = ''
+
         # Set skip_broken (in dnf this is strict=0)
         if self.skip_broken:
             conf.strict = 0
@@ -612,6 +680,9 @@ class DnfModule(YumDnf):
             if self.download_dir:
                 conf.destdir = self.download_dir
 
+        if self.cacheonly:
+            conf.cacheonly = True
+
         # Default in dnf upstream is true
         conf.clean_requirements_on_remove = self.autoremove
 
@@ -635,10 +706,10 @@ class DnfModule(YumDnf):
                 for repo in repos.get_matching(repo_pattern):
                     repo.enable()
 
-    def _base(self, conf_file, disable_gpg_check, disablerepo, enablerepo, installroot):
+    def _base(self, conf_file, disable_gpg_check, disablerepo, enablerepo, installroot, sslverify):
         """Return a fully configured dnf Base object."""
         base = dnf.Base()
-        self._configure_base(base, conf_file, disable_gpg_check, installroot)
+        self._configure_base(base, conf_file, disable_gpg_check, installroot, sslverify)
         try:
             # this method has been supported in dnf-4.2.17-6 or later
             # https://bugzilla.redhat.com/show_bug.cgi?id=1788212
@@ -674,15 +745,25 @@ class DnfModule(YumDnf):
                 rc=1
             )
 
-        filters = []
-        if self.bugfix:
-            key = {'advisory_type__eq': 'bugfix'}
-            filters.append(base.sack.query().upgrades().filter(**key))
-        if self.security:
-            key = {'advisory_type__eq': 'security'}
-            filters.append(base.sack.query().upgrades().filter(**key))
-        if filters:
-            base._update_security_filters = filters
+        add_security_filters = getattr(base, "add_security_filters", None)
+        if callable(add_security_filters):
+            filters = {}
+            if self.bugfix:
+                filters.setdefault('types', []).append('bugfix')
+            if self.security:
+                filters.setdefault('types', []).append('security')
+            if filters:
+                add_security_filters('eq', **filters)
+        else:
+            filters = []
+            if self.bugfix:
+                key = {'advisory_type__eq': 'bugfix'}
+                filters.append(base.sack.query().upgrades().filter(**key))
+            if self.security:
+                key = {'advisory_type__eq': 'security'}
+                filters.append(base.sack.query().upgrades().filter(**key))
+            if filters:
+                base._update_security_filters = filters
 
         return base
 
@@ -798,7 +879,7 @@ class DnfModule(YumDnf):
         except dnf.exceptions.DepsolveError as e:
             return {
                 'failed': True,
-                'msg': "Depsolve Error occured for package {0}.".format(pkg_spec),
+                'msg': "Depsolve Error occurred for package {0}.".format(pkg_spec),
                 'failure': " ".join((pkg_spec, to_native(e))),
                 'rc': 1,
                 "results": []
@@ -810,15 +891,19 @@ class DnfModule(YumDnf):
             else:
                 return {
                     'failed': True,
-                    'msg': "Unknown Error occured for package {0}.".format(pkg_spec),
+                    'msg': "Unknown Error occurred for package {0}.".format(pkg_spec),
                     'failure': " ".join((pkg_spec, to_native(e))),
                     'rc': 1,
                     "results": []
                 }
 
     def _whatprovides(self, filepath):
+        self.base.read_all_repos()
         available = self.base.sack.query().available()
-        pkg_spec = available.filter(provides=filepath).run()
+        # Search in file
+        files_filter = available.filter(file=filepath)
+        # And Search in provides
+        pkg_spec = files_filter.union(available.filter(provides=filepath)).run()
 
         if pkg_spec:
             return pkg_spec[0].name
@@ -833,14 +918,13 @@ class DnfModule(YumDnf):
                 filenames.append(name)
             elif name.endswith(".rpm"):
                 filenames.append(name)
-            elif name.startswith("@") or ('/' in name):
+            elif name.startswith('/'):
                 # like "dnf install /usr/bin/vi"
-                if '/' in name:
-                    pkg_spec = self._whatprovides(name)
-                    if pkg_spec:
-                        pkg_specs.append(pkg_spec)
-                        continue
-
+                pkg_spec = self._whatprovides(name)
+                if pkg_spec:
+                    pkg_specs.append(pkg_spec)
+                    continue
+            elif name.startswith("@") or ('/' in name):
                 if not already_loaded_comps:
                     self.base.read_comps()
                     already_loaded_comps = True
@@ -870,7 +954,7 @@ class DnfModule(YumDnf):
                         self.base.package_upgrade(pkg)
                 except Exception as e:
                     self.module.fail_json(
-                        msg="Error occured attempting update_only operation: {0}".format(to_native(e)),
+                        msg="Error occurred attempting update_only operation: {0}".format(to_native(e)),
                         results=[],
                         rc=1,
                     )
@@ -890,7 +974,7 @@ class DnfModule(YumDnf):
             except IOError as e:
                 if to_text("Can not load RPM file") in to_text(e):
                     self.module.fail_json(
-                        msg="Error occured attempting remote rpm install of package: {0}. {1}".format(filename, to_native(e)),
+                        msg="Error occurred attempting remote rpm install of package: {0}. {1}".format(filename, to_native(e)),
                         results=[],
                         rc=1,
                     )
@@ -906,7 +990,7 @@ class DnfModule(YumDnf):
                         self.base.package_install(pkg)
                 except Exception as e:
                     self.module.fail_json(
-                        msg="Error occured attempting remote rpm operation: {0}".format(to_native(e)),
+                        msg="Error occurred attempting remote rpm operation: {0}".format(to_native(e)),
                         results=[],
                         rc=1,
                     )
@@ -956,7 +1040,7 @@ class DnfModule(YumDnf):
             try:
                 self.base.upgrade_all()
             except dnf.exceptions.DepsolveError as e:
-                failure_response['msg'] = "Depsolve Error occured attempting to upgrade all packages"
+                failure_response['msg'] = "Depsolve Error occurred attempting to upgrade all packages"
                 self.module.fail_json(**failure_response)
         else:
             pkg_specs, group_specs, module_specs, filenames = self._parse_spec_group_file()
@@ -1005,7 +1089,7 @@ class DnfModule(YumDnf):
                         else:
                             response['results'].append("Group {0} installed.".format(group))
                     except dnf.exceptions.DepsolveError as e:
-                        failure_response['msg'] = "Depsolve Error occured attempting to install group: {0}".format(group)
+                        failure_response['msg'] = "Depsolve Error occurred attempting to install group: {0}".format(group)
                         self.module.fail_json(**failure_response)
                     except dnf.exceptions.Error as e:
                         # In dnf 2.0 if all the mandatory packages in a group do
@@ -1017,7 +1101,7 @@ class DnfModule(YumDnf):
                     try:
                         self.base.environment_install(environment, dnf.const.GROUP_PACKAGE_TYPES)
                     except dnf.exceptions.DepsolveError as e:
-                        failure_response['msg'] = "Depsolve Error occured attempting to install environment: {0}".format(environment)
+                        failure_response['msg'] = "Depsolve Error occurred attempting to install environment: {0}".format(environment)
                         self.module.fail_json(**failure_response)
                     except dnf.exceptions.Error as e:
                         failure_response['failures'].append(" ".join((environment, to_native(e))))
@@ -1085,7 +1169,7 @@ class DnfModule(YumDnf):
                             # If not already installed, try to install.
                             self.base.environment_install(environment, dnf.const.GROUP_PACKAGE_TYPES)
                     except dnf.exceptions.DepsolveError as e:
-                        failure_response['msg'] = "Depsolve Error occured attempting to install environment: {0}".format(environment)
+                        failure_response['msg'] = "Depsolve Error occurred attempting to install environment: {0}".format(environment)
                     except dnf.exceptions.Error as e:
                         failure_response['failures'].append(" ".join((environment, to_native(e))))
 
@@ -1234,7 +1318,7 @@ class DnfModule(YumDnf):
                             fail = True
 
                         if fail:
-                            msg = 'Failed to validate GPG signature for {0}'.format(package)
+                            msg = 'Failed to validate GPG signature for {0}: {1}'.format(package, gpgerr)
                             self.module.fail_json(msg)
 
                 if self.download_only:
@@ -1242,14 +1326,18 @@ class DnfModule(YumDnf):
                     # Just return them.
                     self.module.exit_json(**response)
                 else:
-                    self.base.do_transaction()
+                    tid = self.base.do_transaction()
+                    if tid is not None:
+                        transaction = self.base.history.old([tid])[0]
+                        if transaction.return_code:
+                            failure_response['failures'].append(transaction.output())
 
                 if failure_response['failures']:
                     failure_response['msg'] = 'Failed to install some of the specified packages'
-                    self.module.exit_json(**response)
+                    self.module.fail_json(**failure_response)
                 self.module.exit_json(**response)
         except dnf.exceptions.DepsolveError as e:
-            failure_response['msg'] = "Depsolve Error occured: {0}".format(to_native(e))
+            failure_response['msg'] = "Depsolve Error occurred: {0}".format(to_native(e))
             self.module.fail_json(**failure_response)
         except dnf.exceptions.Error as e:
             if to_text("already installed") in to_text(e):
@@ -1257,13 +1345,9 @@ class DnfModule(YumDnf):
                 response['results'].append("Package already installed: {0}".format(to_native(e)))
                 self.module.exit_json(**response)
             else:
-                failure_response['msg'] = "Unknown Error occured: {0}".format(to_native(e))
+                failure_response['msg'] = "Unknown Error occurred: {0}".format(to_native(e))
                 self.module.fail_json(**failure_response)
 
-    @staticmethod
-    def has_dnf():
-        return HAS_DNF
-
     def run(self):
         """The main function."""
 
@@ -1286,7 +1370,7 @@ class DnfModule(YumDnf):
         if self.update_cache and not self.names and not self.list:
             self.base = self._base(
                 self.conf_file, self.disable_gpg_check, self.disablerepo,
-                self.enablerepo, self.installroot
+                self.enablerepo, self.installroot, self.sslverify
             )
             self.module.exit_json(
                 msg="Cache updated",
@@ -1304,20 +1388,20 @@ class DnfModule(YumDnf):
         if self.list:
             self.base = self._base(
                 self.conf_file, self.disable_gpg_check, self.disablerepo,
-                self.enablerepo, self.installroot
+                self.enablerepo, self.installroot, self.sslverify
             )
             self.list_items(self.list)
         else:
             # Note: base takes a long time to run so we want to check for failure
             # before running it.
-            if not dnf.util.am_i_root():
+            if not self.download_only and not dnf.util.am_i_root():
                 self.module.fail_json(
                     msg="This command has to be run under the root user.",
                     results=[],
                 )
             self.base = self._base(
                 self.conf_file, self.disable_gpg_check, self.disablerepo,
-                self.enablerepo, self.installroot
+                self.enablerepo, self.installroot, self.sslverify
             )
 
             if self.with_modules:
