@@ -1,4 +1,3 @@
-#!/usr/bin/python
 # -*- coding: utf-8 -*-
 
 # Copyright: (c) 2012, Matt Wright <matt@nobien.net>
@@ -142,97 +141,96 @@ author:
 
 EXAMPLES = '''
 - name: Install bottle python package
-  pip:
+  ansible.builtin.pip:
     name: bottle
 
 - name: Install bottle python package on version 0.11
-  pip:
+  ansible.builtin.pip:
     name: bottle==0.11
 
 - name: Install bottle python package with version specifiers
-  pip:
+  ansible.builtin.pip:
     name: bottle>0.10,<0.20,!=0.11
 
 - name: Install multi python packages with version specifiers
-  pip:
+  ansible.builtin.pip:
     name:
       - django>1.11.0,<1.12.0
       - bottle>0.10,<0.20,!=0.11
 
 - name: Install python package using a proxy
-  # Pip doesn't use the standard environment variables, please use the CAPITALIZED ones below
-  pip:
+  ansible.builtin.pip:
     name: six
   environment:
-    HTTP_PROXY: '127.0.0.1:8080'
-    HTTPS_PROXY: '127.0.0.1:8080'
+    http_proxy: 'http://127.0.0.1:8080'
+    https_proxy: 'https://127.0.0.1:8080'
 
 # You do not have to supply '-e' option in extra_args
 - name: Install MyApp using one of the remote protocols (bzr+,hg+,git+,svn+)
-  pip:
+  ansible.builtin.pip:
     name: svn+http://myrepo/svn/MyApp#egg=MyApp
 
 - name: Install MyApp using one of the remote protocols (bzr+,hg+,git+)
-  pip:
+  ansible.builtin.pip:
     name: git+http://myrepo/app/MyApp
 
 - name: Install MyApp from local tarball
-  pip:
+  ansible.builtin.pip:
     name: file:///path/to/MyApp.tar.gz
 
 - name: Install bottle into the specified (virtualenv), inheriting none of the globally installed modules
-  pip:
+  ansible.builtin.pip:
     name: bottle
     virtualenv: /my_app/venv
 
 - name: Install bottle into the specified (virtualenv), inheriting globally installed modules
-  pip:
+  ansible.builtin.pip:
     name: bottle
     virtualenv: /my_app/venv
     virtualenv_site_packages: yes
 
 - name: Install bottle into the specified (virtualenv), using Python 2.7
-  pip:
+  ansible.builtin.pip:
     name: bottle
     virtualenv: /my_app/venv
     virtualenv_command: virtualenv-2.7
 
 - name: Install bottle within a user home directory
-  pip:
+  ansible.builtin.pip:
     name: bottle
     extra_args: --user
 
 - name: Install specified python requirements
-  pip:
+  ansible.builtin.pip:
     requirements: /my_app/requirements.txt
 
 - name: Install specified python requirements in indicated (virtualenv)
-  pip:
+  ansible.builtin.pip:
     requirements: /my_app/requirements.txt
     virtualenv: /my_app/venv
 
 - name: Install specified python requirements and custom Index URL
-  pip:
+  ansible.builtin.pip:
     requirements: /my_app/requirements.txt
     extra_args: -i https://example.com/pypi/simple
 
 - name: Install specified python requirements offline from a local directory with downloaded packages
-  pip:
+  ansible.builtin.pip:
     requirements: /my_app/requirements.txt
     extra_args: "--no-index --find-links=file:///my_downloaded_packages_dir"
 
 - name: Install bottle for Python 3.3 specifically, using the 'pip3.3' executable
-  pip:
+  ansible.builtin.pip:
     name: bottle
     executable: pip3.3
 
 - name: Install bottle, forcing reinstallation if it's already installed
-  pip:
+  ansible.builtin.pip:
     name: bottle
     state: forcereinstall
 
 - name: Install bottle while ensuring the umask is 0022 (to ensure other users can use it)
-  pip:
+  ansible.builtin.pip:
     name: bottle
     umask: "0022"
   become: True
@@ -245,7 +243,7 @@ cmd:
   type: str
   sample: pip2 install ansible six
 name:
-  description: list of python modules targetted by pip
+  description: list of python modules targeted by pip
   returned: success
   type: list
   sample: ['ansible', 'six']
@@ -273,6 +271,8 @@ import tempfile
 import operator
 import shlex
 import traceback
+import types
+
 from ansible.module_utils.compat.version import LooseVersion
 
 SETUPTOOLS_IMP_ERR = None
@@ -455,15 +455,15 @@ def _get_pip(module, env=None, executable=None):
 def _have_pip_module():  # type: () -> bool
     """Return True if the `pip` module can be found using the current Python interpreter, otherwise return False."""
     try:
-        import importlib
+        from importlib.util import find_spec
     except ImportError:
-        importlib = None
+        find_spec = None  # type: ignore[assignment] # type: ignore[no-redef]
 
-    if importlib:
+    if find_spec:
         # noinspection PyBroadException
         try:
             # noinspection PyUnresolvedReferences
-            found = bool(importlib.util.find_spec('pip'))
+            found = bool(find_spec('pip'))
         except Exception:
             found = False
     else:
