@@ -22,9 +22,12 @@ DOCUMENTATION = '''
     notes:
         - Many options default to C(None) here but that only means we do not override the SSH tool's defaults and/or configuration.
           For example, if you specify the port in this plugin it will override any C(Port) entry in your C(.ssh/config).
+        - The ssh CLI tool uses return code 255 as a 'connection error', this can conflict with commands/tools that
+          also return 255 as an error code and will look like an 'unreachable' condition or 'connection error' to this plugin.
     options:
       host:
           description: Hostname/IP to connect to.
+          default: inventory_hostname
           vars:
                - name: inventory_hostname
                - name: ansible_host
@@ -173,7 +176,10 @@ DOCUMENTATION = '''
             - name: ssh_extra_args
           default: ''
       reconnection_retries:
-          description: Number of attempts to connect.
+          description:
+            - Number of attempts to connect.
+            - Ansible retries connections only if it gets an SSH error with a return code of 255.
+            - Any errors with return codes other than 255 indicate an issue with program execution.
           default: 0
           type: integer
           env:
@@ -197,6 +203,8 @@ DOCUMENTATION = '''
           vars:
             - name: ansible_port
             - name: ansible_ssh_port
+          keyword:
+            - name: port
       remote_user:
           description:
               - User name with which to login to the remote server, normally set by the remote_user keyword.
@@ -211,11 +219,15 @@ DOCUMENTATION = '''
             - name: ansible_ssh_user
           cli:
             - name: user
+          keyword:
+            - name: remote_user
       pipelining:
           env:
             - name: ANSIBLE_PIPELINING
             - name: ANSIBLE_SSH_PIPELINING
           ini:
+            - section: defaults
+              key: pipelining
             - section: connection
               key: pipelining
             - section: ssh_connection
@@ -289,9 +301,13 @@ DOCUMENTATION = '''
             - name: ansible_ssh_transfer_method
               version_added: '2.12'
       scp_if_ssh:
+        deprecated:
+              why: In favor of the "ssh_transfer_method" option.
+              version: "2.17"
+              alternatives: ssh_transfer_method
         default: smart
         description:
-          - "Preferred method to use when transfering files over SSH."
+          - "Preferred method to use when transferring files over SSH."
           - When set to I(smart), Ansible will try them until one succeeds or they all fail.
           - If set to I(True), it will force 'scp', if I(False) it will use 'sftp'.
           - This setting will overridden by ssh_transfer_method if set.
@@ -315,7 +331,7 @@ DOCUMENTATION = '''
       timeout:
         default: 10
         description:
-            - This is the default ammount of time we will wait while establishing an SSH connection.
+            - This is the default amount of time we will wait while establishing an SSH connection.
             - It also controls how long we can wait to access reading the connection once established (select on the socket).
         env:
             - name: ANSIBLE_TIMEOUT
@@ -352,6 +368,7 @@ import hashlib
 import os
 import pty
 import re
+import shlex
 import subprocess
 import time
 
@@ -365,7 +382,6 @@ from ansible.errors import (
 from ansible.errors import AnsibleOptionsError
 from ansible.module_utils.compat import selectors
 from ansible.module_utils.six import PY3, text_type, binary_type
-from ansible.module_utils.six.moves import shlex_quote
 from ansible.module_utils._text import to_bytes, to_native, to_text
 from ansible.module_utils.parsing.convert_bool import BOOLEANS, boolean
 from ansible.plugins.connection import ConnectionBase, BUFSIZE
@@ -375,10 +391,12 @@ from ansible.utils.path import unfrackpath, makedirs_safe
 
 display = Display()
 
-
+# error messages that indicate 255 return code is not from ssh itself.
 b_NOT_SSH_ERRORS = (b'Traceback (most recent call last):',  # Python-2.6 when there's an exception
-                                                            # while invoking a script via -m
-                    b'PHP Parse error:',  # Php always returns error 255
+                                                            #   while invoking a script via -m
+                    b'PHP Parse error:',                    # Php always returns with error
+                    b'chmod: invalid mode',                 # chmod, but really only on AIX
+                    b'chmod: A flag or octal number is not correct.',    # chmod, other AIX
                     )
 
 SSHPASS_AVAILABLE = None
@@ -423,7 +441,8 @@ def _handle_error(remaining_retries, command, return_tuple, no_log, host, displa
     if return_tuple[0] == 255:
         SSH_ERROR = True
         for signature in b_NOT_SSH_ERRORS:
-            if signature in return_tuple[1]:
+            # 1 == stout, 2 == stderr
+            if signature in return_tuple[1] or signature in return_tuple[2]:
                 SSH_ERROR = False
                 break
 
@@ -871,7 +890,7 @@ class Connection(ConnectionBase):
         '''
 
         # We don't use _shell.quote as this is run on the controller and independent from the shell plugin chosen
-        display_cmd = u' '.join(shlex_quote(to_text(c)) for c in cmd)
+        display_cmd = u' '.join(shlex.quote(to_text(c)) for c in cmd)
         display.vvv(u'SSH: EXEC {0}'.format(display_cmd), host=self.host)
 
         # Start the given command. If we don't need to pipeline data, we can try
@@ -1211,7 +1230,7 @@ class Connection(ConnectionBase):
             returncode = stdout = stderr = None
             if method == 'sftp':
                 cmd = self._build_command(self.get_option('sftp_executable'), 'sftp', to_bytes(host))
-                in_data = u"{0} {1} {2}\n".format(sftp_action, shlex_quote(in_path), shlex_quote(out_path))
+                in_data = u"{0} {1} {2}\n".format(sftp_action, shlex.quote(in_path), shlex.quote(out_path))
                 in_data = to_bytes(in_data, nonstring='passthru')
                 (returncode, stdout, stderr) = self._bare_run(cmd, in_data, checkrc=False)
             elif method == 'scp':
