@@ -58,16 +58,16 @@ DOCUMENTATION = """
 
 EXAMPLES = """
 - name: show templating results
-  debug:
-    msg: "{{ lookup('template', './some_template.j2') }}"
+  ansible.builtin.debug:
+    msg: "{{ lookup('ansible.builtin.template', './some_template.j2') }}"
 
 - name: show templating results with different variable start and end string
-  debug:
-    msg: "{{ lookup('template', './some_template.j2', variable_start_string='[%', variable_end_string='%]') }}"
+  ansible.builtin.debug:
+    msg: "{{ lookup('ansible.builtin.template', './some_template.j2', variable_start_string='[%', variable_end_string='%]') }}"
 
 - name: show templating results with different comment start and end string
-  debug:
-    msg: "{{ lookup('template', './some_template.j2', comment_start_string='[#', comment_end_string='#]') }}"
+  ansible.builtin.debug:
+    msg: "{{ lookup('ansible.builtin.template', './some_template.j2', comment_start_string='[#', comment_end_string='#]') }}"
 """
 
 RETURN = """
@@ -80,14 +80,14 @@ _raw:
 from copy import deepcopy
 import os
 
+import ansible.constants as C
+
 from ansible.errors import AnsibleError
 from ansible.plugins.lookup import LookupBase
 from ansible.module_utils._text import to_bytes, to_text
-from ansible.template import generate_ansible_template_vars, AnsibleEnvironment, USE_JINJA2_NATIVE
+from ansible.template import generate_ansible_template_vars, AnsibleEnvironment
 from ansible.utils.display import Display
-
-if USE_JINJA2_NATIVE:
-    from ansible.utils.native_jinja import NativeJinjaText
+from ansible.utils.native_jinja import NativeJinjaText
 
 
 display = Display()
@@ -104,16 +104,16 @@ class LookupModule(LookupBase):
         # capture options
         convert_data_p = self.get_option('convert_data')
         lookup_template_vars = self.get_option('template_vars')
-        jinja2_native = self.get_option('jinja2_native')
+        jinja2_native = self.get_option('jinja2_native') and C.DEFAULT_JINJA2_NATIVE
         variable_start_string = self.get_option('variable_start_string')
         variable_end_string = self.get_option('variable_end_string')
         comment_start_string = self.get_option('comment_start_string')
         comment_end_string = self.get_option('comment_end_string')
 
-        if USE_JINJA2_NATIVE and not jinja2_native:
-            templar = self._templar.copy_with_new_env(environment_class=AnsibleEnvironment)
-        else:
+        if jinja2_native:
             templar = self._templar
+        else:
+            templar = self._templar.copy_with_new_env(environment_class=AnsibleEnvironment)
 
         for term in terms:
             display.debug("File lookup term: %s" % term)
@@ -153,7 +153,7 @@ class LookupModule(LookupBase):
                     res = templar.template(template_data, preserve_trailing_newlines=True,
                                            convert_data=convert_data_p, escape_backslashes=False)
 
-                if USE_JINJA2_NATIVE and not jinja2_native:
+                if (C.DEFAULT_JINJA2_NATIVE and not jinja2_native) or not convert_data_p:
                     # jinja2_native is true globally but off for the lookup, we need this text
                     # not to be processed by literal_eval anywhere in Ansible
                     res = NativeJinjaText(res)
