@@ -40,6 +40,7 @@ from .host_configs import (
 from .core_ci import (
     AnsibleCoreCI,
     SshKey,
+    VmResource,
 )
 
 from .util import (
@@ -50,6 +51,7 @@ from .util import (
     get_type_map,
     sanitize_host_name,
     sorted_versions,
+    InternalError,
 )
 
 from .util_common import (
@@ -96,6 +98,7 @@ from .connections import (
 )
 
 from .become import (
+    Become,
     Su,
     Sudo,
 )
@@ -109,11 +112,11 @@ TRemoteConfig = t.TypeVar('TRemoteConfig', bound=RemoteConfig)
 @dataclasses.dataclass(frozen=True)
 class Inventory:
     """Simple representation of an Ansible inventory."""
-    host_groups: t.Dict[str, t.Dict[str, t.Dict[str, str]]]
+    host_groups: t.Dict[str, t.Dict[str, t.Dict[str, t.Union[str, int]]]]
     extra_groups: t.Optional[t.Dict[str, t.List[str]]] = None
 
     @staticmethod
-    def create_single_host(name, variables):  # type: (str, t.Dict[str, str]) -> Inventory
+    def create_single_host(name, variables):  # type: (str, t.Dict[str, t.Union[str, int]]) -> Inventory
         """Return an inventory instance created from the given hostname and variables."""
         return Inventory(host_groups=dict(all={name: variables}))
 
@@ -147,7 +150,7 @@ class Inventory:
         inventory_text = inventory_text.strip()
 
         if not args.explain:
-            write_text_file(path, inventory_text)
+            write_text_file(path, inventory_text + '\n')
 
         display.info(f'>>> Inventory\n{inventory_text}', verbosity=3)
 
@@ -294,12 +297,18 @@ class RemoteProfile(SshTargetHostProfile[TRemoteConfig], metaclass=abc.ABCMeta):
 
     def create_core_ci(self, load):  # type: (bool) -> AnsibleCoreCI
         """Create and return an AnsibleCoreCI instance."""
+        if not self.config.arch:
+            raise InternalError(f'No arch specified for config: {self.config}')
+
         return AnsibleCoreCI(
             args=self.args,
-            platform=self.config.platform,
-            version=self.config.version,
-            provider=self.config.provider,
-            suffix='controller' if self.controller else 'target',
+            resource=VmResource(
+                platform=self.config.platform,
+                version=self.config.version,
+                architecture=self.config.arch,
+                provider=self.config.provider,
+                tag='controller' if self.controller else 'target',
+            ),
             load=load,
         )
 
@@ -361,7 +370,7 @@ class DockerProfile(ControllerHostProfile[DockerConfig], SshTargetHostProfile[Do
         setup_sh = bootstrapper.get_script()
         shell = setup_sh.splitlines()[0][2:]
 
-        docker_exec(self.args, self.container_name, [shell], data=setup_sh)
+        docker_exec(self.args, self.container_name, [shell], data=setup_sh, capture=False)
 
     def deprovision(self):  # type: () -> None
         """Deprovision the host after delegation has completed."""
@@ -448,7 +457,7 @@ class NetworkRemoteProfile(RemoteProfile[NetworkRemoteConfig]):
         """Wait for the instance to be ready. Executed before delegation for the controller and after delegation for targets."""
         self.wait_until_ready()
 
-    def get_inventory_variables(self):
+    def get_inventory_variables(self):  # type: () -> t.Dict[str, t.Optional[t.Union[str, int]]]
         """Return inventory variables for accessing this host."""
         core_ci = self.wait_for_instance()
         connection = core_ci.connection
@@ -461,7 +470,7 @@ class NetworkRemoteProfile(RemoteProfile[NetworkRemoteConfig]):
             ansible_user=connection.username,
             ansible_ssh_private_key_file=core_ci.ssh_key.key,
             ansible_network_os=f'{self.config.collection}.{self.config.platform}' if self.config.collection else self.config.platform,
-        )
+        )  # type: t.Dict[str, t.Optional[t.Union[str, int]]]
 
         return variables
 
@@ -483,8 +492,9 @@ class NetworkRemoteProfile(RemoteProfile[NetworkRemoteConfig]):
 
             for dummy in range(1, 90):
                 try:
-                    intercept_python(self.args, self.args.controller_python, cmd, env)
-                except SubprocessError:
+                    intercept_python(self.args, self.args.controller_python, cmd, env, capture=True)
+                except SubprocessError as ex:
+                    display.warning(str(ex))
                     time.sleep(10)
                 else:
                     return
@@ -546,7 +556,7 @@ class PosixRemoteProfile(ControllerHostProfile[PosixRemoteConfig], RemoteProfile
         shell = setup_sh.splitlines()[0][2:]
 
         ssh = self.get_origin_controller_connection()
-        ssh.run([shell], data=setup_sh)
+        ssh.run([shell], data=setup_sh, capture=False)
 
     def get_ssh_connection(self):  # type: () -> SshConnection
         """Return an SSH connection for accessing the host."""
@@ -562,13 +572,15 @@ class PosixRemoteProfile(ControllerHostProfile[PosixRemoteConfig], RemoteProfile
         )
 
         if settings.user == 'root':
-            become = None
+            become = None  # type: t.Optional[Become]
         elif self.config.platform == 'freebsd':
             become = Su()
         elif self.config.platform == 'macos':
             become = Sudo()
         elif self.config.platform == 'rhel':
             become = Sudo()
+        elif self.config.platform == 'ubuntu':
+            become = Sudo()
         else:
             raise NotImplementedError(f'Become support has not been implemented for platform "{self.config.platform}" and user "{settings.user}" is not root.')
 
@@ -672,7 +684,7 @@ class WindowsRemoteProfile(RemoteProfile[WindowsRemoteConfig]):
         """Wait for the instance to be ready. Executed before delegation for the controller and after delegation for targets."""
         self.wait_until_ready()
 
-    def get_inventory_variables(self):
+    def get_inventory_variables(self):  # type: () -> t.Dict[str, t.Optional[t.Union[str, int]]]
         """Return inventory variables for accessing this host."""
         core_ci = self.wait_for_instance()
         connection = core_ci.connection
@@ -686,7 +698,7 @@ class WindowsRemoteProfile(RemoteProfile[WindowsRemoteConfig]):
             ansible_user=connection.username,
             ansible_password=connection.password,
             ansible_ssh_private_key_file=core_ci.ssh_key.key,
-        )
+        )  # type: t.Dict[str, t.Optional[t.Union[str, int]]]
 
         # HACK: force 2016 to use NTLM + HTTP message encryption
         if self.config.version == '2016':
@@ -716,8 +728,9 @@ class WindowsRemoteProfile(RemoteProfile[WindowsRemoteConfig]):
 
             for dummy in range(1, 120):
                 try:
-                    intercept_python(self.args, self.args.controller_python, cmd, env)
-                except SubprocessError:
+                    intercept_python(self.args, self.args.controller_python, cmd, env, capture=True)
+                except SubprocessError as ex:
+                    display.warning(str(ex))
                     time.sleep(10)
                 else:
                     return
